//  Theseus
//  theMDPPOMDP.h -- MDP-POMDP Implementation
//
//  Blai Bonet, Hector Geffner
//  Universidad Simon Bolivar, 1998, 1999, 2000, 2001, 2002

#ifndef _theMDPPOMDP_INCLUDE_
#define _theMDPPOMDP_INCLUDE_

#include <slist>

#include <theseus/thePOMDP.h>


///////////////////////////////////////////////////////////////////////////////
//
// MDP POMDP Class
//

class MDPPOMDPClass : public POMDPClass
{
public:
  // subclass
  class QValueClass
  {
    // private methods
  QValueClass( const QValueClass& QValue );
    
  public:
    int   numTies;
    float value;
    int   *ties;

    // constructor/destructor
    QValueClass( int size ) { value = 0.0; numTies = 0; ties = new int[size]; }
    ~QValueClass( void ) { delete[] ties; }
  };

private:
  // private methods
  MDPPOMDPClass( const MDPPOMDPClass& pomdp );

  float                      QValue( const beliefClass* belief, int action,
				     bool useCache, int extraParameter );
  QValueClass*               bestQValue( const beliefClass* belief, bool useCache, int extraParameter );
  bool                       checkSolved( hashEntryClass* current, slist<hashEntryClass*>& closed,
					  int extraParameter );

  // private virtual methods
  virtual void               algorithm( bool learning, resultClass& result, int extraParameter );

public:
  // constructors
  MDPPOMDPClass();

  // destructors
  virtual ~MDPPOMDPClass( void );

  // virtual methods
  virtual void               initRun( int run );
  virtual void               endRun( bool lastRun );
  virtual float              cost( int state, int action ) const;
  virtual float              cost( const beliefClass *belief, int action ) const;
  virtual bool               inGoal( int state ) const;
  virtual bool               inGoal( const beliefClass* belief ) const;
  virtual bool               applicable( int state, int action ) const;
  virtual bool               applicable( const beliefClass* belief, int action ) const;
  virtual int                getInitialState( const beliefClass* belief ) const;
  virtual int                nextState( int state, int action ) const;
  virtual int                nextObservation( int state, int action ) const;

  virtual hashValueClass*    hashValue( const beliefClass* belief );
  virtual int                getBestAction( const beliefClass *belief );
  virtual beliefClass*       getBeliefAfterAction( const beliefClass *belief, int action ) const;
  virtual beliefClass*       getBeliefAfterActionObservation( const beliefClass *belief_a,
							      int action, int observation ) const;

  // serialization
  static  void               checkIn( void );
  virtual void               write( ostream& os ) const;
  static MDPPOMDPClass*      read( istream& is );
  static MDPPOMDPClass*      constructor( void );
  static void                fill( istream& is, MDPPOMDPClass *pomdp );
};


#endif // _theMDPPOMDP_INCLUDE
