//  Theseus
//  theRtHash.h -- RunTime State and Observation Hash
//
//  Blai Bonet, Hector Geffner
//  Universidad Simon Bolivar, 1998, 1999, 2000, 2001, 2002

#ifndef _theRtHash_INCLUDE
#define _theRtHash_INCLUDE


#include <theseus/theHash.h>


class stateHashClass : public hashClass
{
  // members
  int   size;
  char* buff;

  // private methods
  stateHashClass( const stateHashClass& hash );

public:
  // constructors/destructors
  stateHashClass();
  virtual ~stateHashClass();

  // methods
  void setSize( int pSize );

  // hash functions (since this is a hashClass derived class)
  virtual unsigned   dataFunction( const void *data );
  virtual void*      dataCopy( const void *data );
  virtual bool       dataCompare( const void *data1, const void *data2 );
  virtual void       dataDelete( void *data );
  virtual void       dataPrint( ostream& os, const void *data );

  // serialization
  virtual hashClass* constructor( void );
  virtual void*      dataRead( istream& is );
  virtual void       dataWrite( ostream& os, const void *data ) const;
};


class invStateHashClass : public hashClass
{
  // private methods
  invStateHashClass( const invStateHashClass& hash );

public:
  // constructors/destructors
  invStateHashClass();
  virtual ~invStateHashClass();

  // hash functions (since this is a hashClass derived class)
  virtual unsigned   dataFunction( const void *data );
  virtual void*      dataCopy( const void *data );
  virtual bool       dataCompare( const void *data1, const void *data2 );
  virtual void       dataDelete( void *data );
  virtual void       dataPrint( ostream& os, const void *data );

  // serialization
  virtual hashClass* constructor( void );
  virtual void*      dataRead( istream& is );
  virtual void       dataWrite( ostream& os, const void *data ) const;
};


class observationHashClass : public hashClass
{
  // members
  int   size;
  char* buff;

  // private methods
  observationHashClass( const observationHashClass& hash );

public:
  // constructors/destructors
  observationHashClass();
  virtual ~observationHashClass();

  // methods
  void setSize( int pSize );

  // hash functions (since this is a hashClass derived class)
  virtual unsigned   dataFunction( const void *data );
  virtual void*      dataCopy( const void *data );
  virtual bool       dataCompare( const void *data1, const void *data2 );
  virtual void       dataDelete( void *data );
  virtual void       dataPrint( ostream& os, const void *data );

  // serialization
  virtual hashClass* constructor( void );
  virtual void*      dataRead( istream& is );
  virtual void       dataWrite( ostream& os, const void *data ) const;
};


class invObservationHashClass : public hashClass
{
  // private methods
  invObservationHashClass( const invObservationHashClass& hash );

public:
  // constructors/destructors
  invObservationHashClass();
  virtual ~invObservationHashClass();

  // hash functions (since this is a hashClass derived class)
  virtual unsigned   dataFunction( const void *data );
  virtual void*      dataCopy( const void *data );
  virtual bool       dataCompare( const void *data1, const void *data2 );
  virtual void       dataDelete( void *data );
  virtual void       dataPrint( ostream& os, const void *data );

  // serialization
  virtual hashClass* constructor( void );
  virtual void*      dataRead( istream& is );
  virtual void       dataWrite( ostream& os, const void *data ) const;
};


#endif   // _theRtHash_INCLUDE
