//  Theseus
//  theSearchBelief.h -- Search Belief Implementation
//
//  Blai Bonet, Hector Geffner
//  Universidad Simon Bolivar, 1998, 1999, 2000, 2001, 2002

#ifndef _theSearchBelief_INCLUDE_
#define _theSearchBelief_INCLUDE_

#include <iostream>
#include <set>
#include <map>

#include <theseus/theBelief.h>


class modelClass;
class searchModelClass;
class beliefCacheClass;


///////////////////////////////////////////////////////////////////////////////
//
// Search Belief Class
//

class searchBeliefClass : public beliefClass
{
private:
  // members
  set<int> bel;

  // static members
  static searchModelClass* model;

  // private methods
  searchBeliefClass( const searchBeliefClass& belief );

  // private virtual methods
  virtual void print( ostream& os ) const;

public:
  // constructors/destructors
  searchBeliefClass();
  virtual ~searchBeliefClass( void );

  // methods
  const searchModelClass*       getModel( void ) const;
  void                          clean( void );

  // virtual methods
  virtual void                  setModel( modelClass* model );
  virtual beliefClass::constructor_t getConstructor( void ) const;
  virtual bool                  check( void ) const;
  virtual bool                  check( int state ) const;
  virtual void                  checkModelAvailability( modelClass* model ) const;
  virtual const map<int,float>* cast( void ) const;
  virtual void                  insert( int state, float probability );
  virtual unsigned              randomSampling( void ) const;
  virtual void                  nextPossibleObservations( int action, map<int,float> &result ) const;
  virtual int                   plausibleState( int action, int observation ) const;
  virtual beliefClass*          updateByA( beliefCacheClass* cache, bool useCache, int action ) const;
  virtual beliefClass*          updateByAO( beliefCacheClass* cache, bool useCache, int action, int observation ) const;
  virtual beliefClass*          clone( void ) const;
  virtual float                 heuristicValue( void ) const;
  virtual void                  support( set<int> &result ) const;
  virtual int                   supportSize( void ) const;

  // operator overload
  virtual beliefClass&          operator=( const beliefClass& bel );
  virtual beliefClass&          operator=( const searchBeliefClass& bel );
  virtual bool                  operator==( const beliefClass& bel ) const;
  virtual bool                  operator==( const searchBeliefClass& bel ) const;

  // hash virtual methods
  virtual unsigned              hashDataFunction( void ) const;
  virtual void*                 hashDataCopy( void ) const;
  virtual bool                  hashDataCompare( const void* data ) const;
  virtual void                  hashDataWrite( ostream& os ) const;
  virtual void*                 hashDataRead( istream& is ) const;

  // serialization
  static  void                  checkIn( void );
  virtual void                  write( ostream& os ) const;
  static  searchBeliefClass*    read( istream& is );
  static  searchBeliefClass*    constructor( void );
  static  void                  fill( istream& is, beliefClass* bel );
  static  void                  fill( istream& is, searchBeliefClass* bel );

  // friends
  friend class searchBeliefHashClass;
  friend class searchPOMDPClass;
  friend class searchModelClass;
};


///////////////////////////////////////////////////////////////////////////////
//
// Search Belief Hash Class
//

class searchBeliefHashClass : public beliefHashClass
{
private:
  // private methods
  searchBeliefHashClass( const searchBeliefHashClass& hash );

public:
  // constructors/destructors
  searchBeliefHashClass();
  virtual ~searchBeliefHashClass();

  // virtual hash functions (since this is a hashClass derived class)
  virtual unsigned               dataFunction( const void* data );
  virtual void*                  dataCopy( const void* data );
  virtual bool                   dataCompare( const void* newData, const void* dataInHash );
  virtual void                   dataDelete( void* data );
  virtual void                   dataPrint( ostream& os, const void* data );
  virtual void                   dataWrite( ostream& os, const void* data ) const;
  virtual void*                  dataRead( istream& is );
  virtual void                   statistics( ostream& os );

  // virtual quantization functions
  virtual void                   quantize( bool useCache, bool saveInHash, const beliefClass* belief,
					   hashEntryClass*& quantization );

  // serialization
  static  void                   checkIn( void );
  virtual void                   write( ostream& os ) const;
  static  searchBeliefHashClass* read( istream& is );
  static  searchBeliefHashClass* constructor( void );
  static  void                   fill( istream& is, searchBeliefHashClass* hash );
};


#endif // _theSearchBelief_INCLUDE
