//  Theseus
//  theStandardBelief.h -- Standard Belief Implementation
//
//  Blai Bonet, Hector Geffner
//  Universidad Simon Bolivar, 1998, 1999, 2000, 2001, 2002

#ifndef _theStandardBelief_INCLUDE_
#define _theStandardBelief_INCLUDE_

#include <iostream>
#include <map>

#include <theseus/theBelief.h>


class quantizationClass;
class standardModelClass;


///////////////////////////////////////////////////////////////////////////////
//
// Standard Belief Class
//

class standardBeliefClass : public beliefClass
{
 private:
  // members
  map<int,float> bel;

  // static members
  static standardModelClass* model;

  // private methods
  standardBeliefClass( const standardBeliefClass& belief );

  // private virtual methods
  virtual void print( ostream& os ) const;

public:
  // constructors/destructors
  standardBeliefClass();
  virtual ~standardBeliefClass( void );

  // methods
  const standardModelClass*     getModel( void ) const;
  void                          clean( void );
  bool                          compact( void );
  void                          normalize( void );
  void                          fill( const float* array, int dimension );

  // virtual methods
  virtual void                  setModel( modelClass* model );
  virtual beliefClass::constructor_t getConstructor( void ) const;
  virtual bool                  check( void ) const;
  virtual bool                  check( int state ) const;
  virtual const map<int,float>* cast( void ) const;
  virtual void                  insert( int state, float probability );
  virtual void                  checkModelAvailability( modelClass* model ) const;
  virtual unsigned              randomSampling( void ) const;
  virtual void                  nextPossibleObservations( int action, map<int,float> &result ) const;
  virtual int                   plausibleState( int action, int observation ) const;
  virtual beliefClass*          updateByA( beliefCacheClass* cache, bool useCache, int action ) const;
  virtual beliefClass*          updateByAO( beliefCacheClass* cache, bool useCache, int action, int observation ) const;
  virtual beliefClass*          clone( void ) const;
  virtual float                 heuristicValue( void ) const;
  virtual void                  support( set<int> &result ) const;
  virtual int                   supportSize( void ) const;

  // operator overload
  virtual beliefClass&          operator=( const beliefClass& bel );
  virtual beliefClass&          operator=( const standardBeliefClass& bel );
  virtual bool                  operator==( const beliefClass& bel ) const;
  virtual bool                  operator==( const standardBeliefClass& bel ) const;

  // hash virtual methods
  virtual unsigned              hashDataFunction( void ) const;
  virtual void*                 hashDataCopy( void ) const;
  virtual bool                  hashDataCompare( const void* data ) const;
  virtual void                  hashDataWrite( ostream& os ) const;
  virtual void*                 hashDataRead( istream& is ) const;

  // serialization
  static  void                  checkIn( void );
  virtual void                  write( ostream& os ) const;
  static  standardBeliefClass*  read( istream& is );
  static  standardBeliefClass*  constructor( void );
  static  void                  fill( istream& is, beliefClass* bel );
  static  void                  fill( istream& is, standardBeliefClass* bel );

  // friends
  friend class standardBeliefHashClass;
};


///////////////////////////////////////////////////////////////////////////////
//
// Standard Belief Hash Class
//

class standardBeliefHashClass : public beliefHashClass
{
  // private methods
  standardBeliefHashClass( const standardBeliefHashClass& hash );

public:
  // members
  quantizationClass*  quantization;

  // constructors/destructors
  standardBeliefHashClass();
  virtual ~standardBeliefHashClass( void );

  // virtual hash functions (since this is a hashClass derived class)
  virtual void                     clean( void );
  virtual unsigned                 dataFunction( const void* data );
  virtual void*                    dataCopy( const void* data );
  virtual bool                     dataCompare( const void* newData, const void* dataInHash );
  virtual void                     dataDelete( void* data );
  virtual void                     dataPrint( ostream& os, const void* data );
  virtual void                     dataWrite( ostream& os, const void* data ) const;
  virtual void*                    dataRead( istream& is );
  virtual void                     statistics( ostream& os );

  // virtual quantization functions
  virtual void                     quantize( bool useCache, bool saveInHash, const beliefClass* belief, 
					     hashEntryClass*& quantization );

  // serialization
  static  void                     checkIn( void );
  virtual void                     write( ostream& os ) const;
  static  standardBeliefHashClass* read( istream& is );
  static  standardBeliefHashClass* constructor( void );
  static  void                     fill( istream& is, standardBeliefHashClass* hash );
};


#endif // _theStandardBelief_INCLUDE
