//  Theseus
//  theStandardModel.h -- Standard Model Implementation
//
//  Blai Bonet, Hector Geffner
//  Universidad Simon Bolivar, 1998, 1999, 2000, 2001, 2002

#ifndef _theStandardModel_INCLUDE_
#define _theStandardModel_INCLUDE_

#include <iostream>
#include <set>
#include <map>

#include <theseus/theModel.h>


class stateClass;
class standardBeliefClass;
class stateHashClass;
class invStateHashClass;
class observationHashClass;
class invObservationHashClass;


///////////////////////////////////////////////////////////////////////////////
//
// Standard Model Class
//

class standardModelClass : public modelClass
{
  // private members
  stateHashClass*            stateHash;
  invStateHashClass*         invStateHash;             
  observationHashClass*      observationHash;
  invObservationHashClass*   invObservationHash;             

  // private methods
  standardModelClass( const standardModelClass& model );

public:
  // members
  int  stateIndex;
  int  observationIndex;
  int  numInitialStates;

  // simple models
  set<int>         availabilityModel;
  set<int>*        applicationModel;
  set<int>         goalModel;
  map<int,float>*  costModel;

  // complex models
  map<int,map<int,float> >*  transitionModel;
  map<int,map<int,float> >*  observationModel;
  map<int,set<int> >         invTransitionModel;

  // constructors/destructors
  standardModelClass( void );
  virtual ~standardModelClass( void );

  // methods
  void                        setup( char *cassandraFilename );
  void                        computeWholeModelFrom( const beliefClass *belief );
  void                        computeWholeModelFrom( int stateIndex );
  void                        printModel( ostream &os ) const;
  void                        setAvailability( int stateIndex );
  const stateClass*           getState( int index ) const;

  // virtual functions
  virtual void                setup( problemHandleClass *handle );
  virtual void                printState( int state, ostream& os, int indent ) const;
  virtual void                printObservation( int obs, ostream& os, int indent ) const;
  virtual void                printData( ostream& os, const char *prefix = NULL ) const;
  virtual void                checkModelFor( int stateIndex );
  virtual void                computeModelFor( int stateIndex, list<int> &newStates );
  virtual int                 getInitialState( const beliefClass *belief ) const;
  virtual float               cost( int state, int action ) const;
  virtual float               cost( const beliefClass *belief, int action ) const;
  virtual bool                applicable( int state, int action ) const;
  virtual bool                applicable( const beliefClass* belief, int action ) const;
  virtual bool                inGoal( int state ) const;
  virtual bool                inGoal( const beliefClass *belief ) const;
  virtual int                 currentNumberStates() const { return stateIndex; }
  virtual int                 currentNumberObservations() const { return observationIndex; }
  virtual int                 nextState( int state, int action ) const;
  virtual int                 nextObservation( int state, int action ) const;
  virtual int                 newState( void );
  virtual void                newTransition( int state, int action, int statePrime, float probability );

  // serialization
  static  void                checkIn( void );
  virtual void                write( ostream& os ) const;
  static  standardModelClass* read( istream& is );
  static  standardModelClass* constructor( void );
  static  void                fill( istream& is, standardModelClass *model );
};


#endif // _theStandardModel_INCLUDE
