//  Theseus
//  theStandardPOMDP.h -- Standard POMDP Implementation
//
//  Blai Bonet, Hector Geffner
//  Universidad Simon Bolivar, 1998, 1999, 2000, 2001, 2002

#ifndef _theStandardPOMDP_INCLUDE_
#define _theStandardPOMDP_INCLUDE_

#include <iostream>
#include <map>
#include <slist>

#include <theseus/thePOMDP.h>


///////////////////////////////////////////////////////////////////////////////
//
// Standard POMDP Class
//

class standardPOMDPClass : public POMDPClass
{
public:
  // subclass
  class QValueClass
  {
    // private methods
    QValueClass( const QValueClass& QValue );

  public:
    int   numTies;
    float value;
    int   *ties;

    // constructor/destructor
    QValueClass( int size )
      { value = 0.0; numTies = 0; ties = new int[size]; }
    ~QValueClass( void )
      { delete[] ties; }
  };

private:
  // private methods
  standardPOMDPClass( const standardPOMDPClass& pomdp );

  float                      QValue( const beliefClass* belief, int action,
				     bool useCache, int extraParameter );
  QValueClass*               bestQValue( const beliefClass* belief, bool useCache, int extraParameter );
  QValueClass*               bestQValue( const hashEntryClass* qBelief, bool useCache, int extraParameter );
  bool                       checkSolved( hashEntryClass* current, slist<hashEntryClass*>& closed,
					  int extraParameter );

  // private virtual methods
  virtual void               algorithm( bool learning, resultClass& result, int extraParameter );

  // transition graph
  map<const hashEntryClass*,map<pair<int,int>,hashEntryClass*>*> trCache;

public:
  // static members
  static int                 numberStates;

  // constructors
  standardPOMDPClass();

  // destructors
  virtual ~standardPOMDPClass( void );

  // static methods
  static void                setup( int numberStates );

  // virtual methods
  virtual const beliefClass* getTheInitialBelief( void );
  virtual void               initialize( void );
  virtual void               finalize( void );
  virtual void               initRun( int run );
  virtual void               endRun( bool lastRun );
  virtual float              cost( int state, int action ) const;
  virtual float              cost( const beliefClass *belief, int action ) const;
  virtual bool               inGoal( int state ) const;
  virtual bool               inGoal( const beliefClass* belief ) const;
  virtual bool               applicable( int state, int action ) const;
  virtual bool               applicable( const beliefClass* belief, int action ) const;
  virtual int                getInitialState( const beliefClass* belief ) const;
  virtual int                nextState( int state, int action ) const;
  virtual int                nextObservation( int state, int action ) const;

  virtual hashValueClass*    hashValue( const beliefClass* belief );
  virtual int                getBestAction( const beliefClass *belief );
  virtual const beliefClass* getBeliefAfterAction( const beliefClass *belief, int action ) const;
  virtual const beliefClass* getBeliefAfterActionObservation( const beliefClass *belief_a,
							      int action, int observation ) const;

  // serialization
  static  void               checkIn( void );
  virtual void               write( ostream& os ) const;
  static standardPOMDPClass* read( istream& is );
  static standardPOMDPClass* constructor( void );
  static void                fill( istream& is, standardPOMDPClass *pomdp );
};


#endif // _theStandardPOMDP_INCLUDE


