//  Theseus
//  theUtils.h -- Utilities Implementation
//
//  Blai Bonet, Hector Geffner
//  Universidad Simon Bolivar, 1998, 1999, 2000, 2001, 2002

#ifndef _theUtils_INCLUDE_
#define _theUtils_INCLUDE_

#include <sys/time.h>
#include <map>
#include <set>
#include <hash_map>
#include <hash_set>


void     safeRead( void *buff, const unsigned size, const unsigned num, istream& is );
void     safeWrite( const void *buff, const unsigned size, const unsigned num, ostream& os );
void     diffTime( unsigned long& secs, unsigned long& usecs, struct timeval& t1, struct timeval& t2 );


unsigned randomSampling( const map<int,float>& m );
unsigned randomSampling( const set<int>& s );
unsigned randomSampling( const hash_map<int,float>& m );
unsigned randomSampling( const hash_set<int>& s );
void     fill( map<int,float>& m, const float *array, int dimension );


template <class T>
ostream& 
operator<<( ostream& os, const set<T>& s )
{
  T elem;
  register unsigned size;
  set<T>::const_iterator it;

  size = s.size();
  safeWrite( &size, sizeof( int ), 1, os );
  for( it = s.begin(); it != s.end(); ++it )
    {
      elem = *it;
      safeWrite( &elem, sizeof( T ), 1, os );
    }
  return( os );
}

template <class T>
ostream& 
operator<<( ostream& os, const hash_set<T>& s )
{
  T elem;
  register unsigned size;
  hash_set<T>::const_iterator it;

  size = s.size();
  safeWrite( &size, sizeof( int ), 1, os );
  for( it = s.begin(); it != s.end(); ++it )
    {
      elem = *it;
      safeWrite( &elem, sizeof( T ), 1, os );
    }
  return( os );
}

template <class T1,class T2>
ostream& 
operator<<( ostream& os, const map<T1,T2>& m )
{
  T1 index;
  T2 elem;
  register unsigned size;
  map<T1,T2>::const_iterator it;

  size = m.size();
  safeWrite( &size, sizeof( int ), 1, os );
  for( it = m.begin(); it != m.end(); ++it )
    {
      index = (*it).first;
      elem = (*it).second;
      safeWrite( &index, sizeof( T1 ), 1, os );
      safeWrite( &elem, sizeof( T2 ), 1, os );
      // os << (*it).first << (*it).second;
    }
  return( os );
}

template <class T1,class T2>
ostream& 
operator<<( ostream& os, const hash_map<T1,T2>& m )
{
  T1 index;
  T2 elem;
  register unsigned size;
  hash_map<T1,T2>::const_iterator it;

  size = m.size();
  safeWrite( &size, sizeof( int ), 1, os );
  for( it = m.begin(); it != m.end(); ++it )
    {
      index = (*it).first;
      elem = (*it).second;
      safeWrite( &index, sizeof( T1 ), 1, os );
      safeWrite( &elem, sizeof( T2 ), 1, os );
    }
  return( os );
}

template <class T>
istream&
operator>>( istream& is, set<T>& s )
{
  T elem;
  register unsigned i, n;

  safeRead( &n, sizeof( int ), 1, is );
  for( i = 0; i < n; ++i )
    {
      safeRead( &elem, sizeof( T ), 1, is );
      s.insert( elem );
    }
  return( is );
}

template <class T>
istream&
operator>>( istream& is, hash_set<T>& s )
{
  T elem;
  register unsigned i, n;

  safeRead( &n, sizeof( int ), 1, is );
  for( i = 0; i < n; ++i )
    {
      safeRead( &elem, sizeof( T ), 1, is );
      s.insert( elem );
    }
  return( is );
}

template <class T1,class T2>
istream&
operator>>( istream& is, map<T1,T2>& m )
{
  T1 index;
  T2 elem;
  register unsigned i, n;

  safeRead( &n, sizeof( int ), 1, is );
  for( i = 0; i < n; ++i )
    {
      safeRead( &index, sizeof( T1 ), 1, is );
      safeRead( &elem, sizeof( T2 ), 1, is );
      m.insert( make_pair( index, elem ) );
    }
  return( is );
}

template <class T1,class T2>
istream&
operator>>( istream& is, hash_map<T1,T2>& m )
{
  T1 index;
  T2 elem;
  register unsigned i, n;

  safeRead( &n, sizeof( int ), 1, is );
  for( i = 0; i < n; ++i )
    {
      safeRead( &index, sizeof( T1 ), 1, is );
      safeRead( &elem, sizeof( T2 ), 1, is );
      m.insert( make_pair( index, elem ) );
    }
  return( is );
}

#endif // _theUtils_INCLUDE
