#ifndef lint
static char const 
yyrcsid[] = "$FreeBSD: src/usr.bin/yacc/skeleton.c,v 1.28 2000/01/17 02:04:06 bde Exp $";
#endif
#include <stdlib.h>
#define YYBYACC 1
#define YYMAJOR 1
#define YYMINOR 9
#define YYLEX yylex()
#define YYEMPTY -1
#define yyclearin (yychar=(YYEMPTY))
#define yyerrok (yyerrflag=0)
#define YYRECOVERING() (yyerrflag!=0)
static int yygrowstack();
#define YYPREFIX "yy"
#line 11 "daeParser.y"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <libgen.h>
#include <assert.h>

#include <daedalus/daeTokens.h>
#include <daedalus/daeTypes.h>
#include <daedalus/daePackage.h>
#include <daedalus/_daeErrors.h>

#include <typeinfo>
#include <list>
#include <map>


#ifdef linux
extern int yylex( void );
#endif


/* globals*/
char                         *yyfile;
char                         *yyparser;
bool                         parsingPredicateId = false;
static const char*           currInternalPredicate = NULL;

static int                   domainType;
static bool                  insideEffect = false; 
static const char            *currentRamification = NULL;
static typeListClass         *plusFunctionDomain;
static typeClass             *plusFunctionRange;
static typeListClass         *minusFunctionDomain;
static typeClass             *minusFunctionRange;

list<const codeChunkClass*>  *globalChunk;
predicateClass               *theEqualPredicate;
predicateClass               *theLessThanPredicate;
predicateClass               *theLessEqualPredicate;
predicateClass               *theGreaterEqualPredicate;
predicateClass               *theGreaterThanPredicate;

booleanTypeClass             *theBooleanType;
genericIntegerTypeClass      *theGenericIntegerType;

domainClass                  *declaredDomain;
problemClass                 *declaredProblem;
actionClass                  *declaredAction;

map<const char*,const domainClass*,ltcasestr> domainHash;
map<const char*,const char*,ltcasestr> problemSet;

map<const char*,const functionClass*,ltcasestr> globalFunctionHash;
map<const char*,const predicateClass*,ltcasestr> globalPredicateHash;
map<const char*,const namedTypeClass*,ltcasestr> globalNamedTypeHash;
map<const char*,const functionClass*,ltcasestr> *functionHash;
map<const char*,const predicateClass*,ltcasestr> *predicateHash;
map<const char*,const namedTypeClass*,ltcasestr> *namedTypeHash;
map<const char*,list<const char*>*,ltcasestr> *objectHashByType;
map<const char*,const typeClass*,ltcasestr> *objectHashByName;
map<const char*,const arrayClass*,ltcasestr> *arrayHash;

set<const char*,ltcasestr>     actionSet;
set<const char*,ltcasestr>     ramificationSet;
set<const char*,ltcasestr>     internalPredicateSet;

list<const actionClass*>       *actionList;
list<const ramificationClass*> *ramificationList;
list<const internalPredicateClass*> *internalPredicateList;

set<const char*,ltcasestr>      ramDirtyPredicates;
set<const char*,ltcasestr>      ramDirtyFunctions;



/* environment for variables*/
static list<map<const char*,const typeClass*,ltcasestr>*> variableEnvironment;
static list<pair<const char*,const typeClass*> > parameterList;

/* prototypes*/
static void                  startDomain( const char* );
static const codeChunkClass* finishDomain( void );
static void                  declareModel( void );
static void                  declareDynamics( int );
static void                  declareFeedback( int );
static void                  declareExternal( const list<const char*> * );
static void                  declareNamedType( const char*, const list<const char*> * );
static void                  declareFunction( const char*, const typeClass*, const typeClass* );
static void                  declarePredicate( const char*, const typeClass* );
static void                  declareSimpleObjects( const char*, const list<const char*> * );
static void                  declareIntegerObjects( const char*, const char*, const list<const char*> * );
static void                  declareBooleanObjects( const list<const char*> * );
static void                  declareArrayObjects( const char*, const typeClass*, const list<const char*> * );
static void                  declareVariable( const char*, const typeClass* );
static void                  startRamification( const char* );
static void                  declareRamification( const formulaClass* );
static void                  declareRamification( const effectClass* );
static void                  startLogicalDef( const char* );
static void                  declareLogicalDef( const char*, const formulaClass* );
static void                  startAction( const char* );
static void                  finishAction( void );
static void                  declareActionPrecondition( const formulaClass* );
static void                  declareActionEffect( const effectClass* );
static void                  declareActionCost( const costClass* );
static void                  declareActionObservation( const observationClass* );
static termClass *           fetchVariableTerm( const char*, termClass* );
static termClass *           fetchObjectTerm( const char*, const termClass* );
static bool                  checkProbabilities( const probabilisticEffectClass* );
static void                  checkProbabilities(const list<pair<const char*,const observationListClass*>*>*);
static void                  startProblem( const char*, const char* );
static const codeChunkClass* finishProblem( void );
static void                  declareInit( const effectListClass* );
static void                  declareGoal( const formulaClass* );
static void                  declareHook( const hookClass* );
static void                  cleanEnvironment( map<const char*,const typeClass*,ltcasestr> * );
static void                  printEnvironment( void ); 
static list<map<const char*,const typeClass*,ltcasestr>*>* environmentClone( void );

extern void                  generateCode( problemClass* );
#line 143 "y.tab.c"
#define YYERRCODE 256
#define DEFINE 257
#define DOMAIN 258
#define CDOMAIN 259
#define PROBLEM 260
#define REQUIREMENTS 261
#define TYPES 262
#define FUNCTIONS 263
#define PREDICATES 264
#define PREDICATE 265
#define EXTERNAL 266
#define OBJECTS 267
#define ACTION 268
#define PARAMETERS 269
#define PRECONDITION 270
#define EFFECT 271
#define COST 272
#define OBSERVATION 273
#define WHEN 274
#define INIT 275
#define GOAL 276
#define PACKAGE 277
#define RAMIFICATION 278
#define FORMULA 279
#define SET 280
#define IN 281
#define ONEOF 282
#define PRINT 283
#define PRINTSTATE 284
#define ASSERT 285
#define HOOKS 286
#define BELIEF 287
#define HEURISTIC 288
#define MODEL 289
#define PLANNING 290
#define CONFORMANT 291
#define ND_MDP 292
#define ND_POMDP 293
#define MDP 294
#define POMDP 295
#define DYNAMICS 296
#define DETERMINISTIC 297
#define NON_DETERMINISTIC 298
#define PROBABILISTIC 299
#define FEEDBACK 300
#define COMPLETE 301
#define PARTIAL 302
#define NULLF 303
#define INTEGER 304
#define BOOLEAN 305
#define ARRAY 306
#define LEFTPAR 307
#define RIGHTPAR 308
#define LEFTSQPAR 309
#define RIGHTSQPAR 310
#define LEFTCURLY 311
#define RIGHTCURLY 312
#define COLON 313
#define STAR 314
#define HYPHEN 315
#define COMMA 316
#define NOT 317
#define AND 318
#define OR 319
#define IF 320
#define IFF 321
#define EQUAL 322
#define LT 323
#define LE 324
#define GE 325
#define GT 326
#define PLUS 327
#define EXISTS 328
#define FORALL 329
#define VECTOR 330
#define SUM 331
#define INLINE 332
#define CERTAINTY 333
#define TOK_IDENT 334
#define TOK_VIDENT 335
#define TOK_INTEGER 336
#define TOK_FLOAT 337
#define TOK_BOOLEAN 338
#define TOK_STRING 339
#define TOK_FUNCTION 340
#define TOK_PREDICATE 341
#define BOGUS 342
#define BOGUS2 343
#define BOGUS3 344
#define BOGUS4 345
const short yylhs[] = {                                        -1,
    0,    3,    3,    3,   49,    1,   50,   50,   54,   54,
   54,   54,   54,   54,   54,   55,   55,   55,   61,   61,
   61,   62,   62,   62,   56,   56,   65,   63,   64,   57,
   57,   67,   67,    5,    5,    4,   58,   58,   68,   59,
   59,   69,    8,    8,    8,    9,    9,    9,   11,   12,
   70,   10,   10,   14,   14,   13,   13,   60,   60,   71,
   71,   71,   71,    6,    6,   51,   51,   72,   75,   76,
   73,   52,   52,   79,   77,   80,   80,   78,   78,   74,
   74,   81,    7,    7,   53,   53,   84,   82,   21,   83,
   83,   85,   85,   86,   85,   85,   85,   15,   17,   17,
   17,   17,   17,   17,   87,   17,   88,   17,   18,   18,
   16,   16,   19,   19,   19,   22,   22,   23,   23,   23,
   23,   26,   26,   27,   27,   27,   27,   27,   27,   89,
   27,   27,   27,   28,   28,   29,   29,   25,   25,   24,
   24,   30,   30,   30,   37,   37,   31,   31,   90,   31,
   38,   38,   32,   32,   32,   32,   32,   32,   34,   34,
   34,   34,   34,   20,   20,   41,   41,   36,   39,   39,
   33,   42,   43,   43,   44,   44,   45,   45,   45,   45,
   91,   45,   47,   47,   46,   93,    2,   92,   92,   94,
   94,   94,   94,   95,   96,   96,   40,   40,   35,   35,
   35,   98,   35,   97,   97,   48,   48,   48,  100,   66,
   99,
};
const short yylen[] = {                                         2,
    1,    2,    2,    0,    0,   13,    3,    2,    3,    3,
    3,    3,    3,    3,    3,    2,    2,    0,    4,    4,
    4,    4,    4,    4,    2,    1,    0,    6,    1,    2,
    1,    2,    1,    2,    1,    3,    2,    1,    5,    2,
    1,    4,    1,    3,    5,    1,    1,    1,    2,    5,
    1,    2,    1,    2,    1,    1,    1,    2,    1,    3,
    3,    3,    7,    2,    1,    3,    0,    1,    0,    0,
    9,    3,    0,    0,    5,    3,    3,    2,    0,    2,
    1,    3,    2,    1,    3,    1,    0,    5,    1,    2,
    0,    2,    2,    0,    3,    2,    2,    1,    1,    4,
    4,    4,    5,    5,    0,    9,    0,    9,    1,    0,
    2,    1,    4,    5,    7,    1,    1,    1,    1,    1,
    1,    2,    0,    1,    2,    1,    4,    4,    2,    0,
    9,    6,    4,    3,    0,    1,    1,    2,    2,    3,
    0,    1,    4,    4,    2,    1,    1,    5,    0,    8,
    2,    1,    5,    8,    8,    4,    4,    1,    9,    7,
   12,    4,    1,    2,    0,    2,    1,    4,    2,    1,
    3,    1,    1,    4,    2,    1,    1,    1,    5,    5,
    0,    8,    2,    1,    4,    0,   13,    2,    0,    4,
    4,    4,    4,    1,    1,    1,    2,    1,    1,    1,
    5,    0,    8,    2,    1,    4,    4,    4,    0,    5,
    1,
};
const short yydefred[] = {                                      4,
    0,    0,    0,    2,    3,    0,    0,    0,    0,    0,
    0,    5,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   18,    0,    0,    0,    0,    0,   26,
    0,    0,   31,    0,    0,   38,    0,    0,   41,   56,
   57,   55,    0,   65,    0,    0,   59,    0,    0,    0,
    8,  186,    0,   10,   25,    0,   35,    0,   11,   30,
    0,   12,   37,    0,   13,   40,   14,   54,    0,   64,
   15,   58,    0,    9,   16,   17,    0,    0,    0,    0,
   68,    0,    7,  189,    0,    0,   34,    0,   51,    0,
    0,   46,    0,   43,   47,   48,   53,    0,    0,   60,
   61,   62,    0,    0,    0,   74,    0,    0,   86,   66,
    0,    0,   27,   36,    0,   49,    0,    0,    0,   42,
   52,    0,    0,    0,    0,    0,    0,    0,   69,    0,
   89,   87,    0,    6,   72,    0,  187,  188,    0,    0,
    0,   44,   39,    0,   19,   20,   21,   22,   23,   24,
    0,    0,    0,   91,   85,    0,    0,    0,    0,    0,
   28,   29,    0,    0,    0,   84,    0,    0,   81,    0,
    0,    0,   75,    0,    0,  158,    0,    0,  163,   99,
  199,  200,  198,    0,    0,    0,  196,  195,    0,    0,
  205,    0,    0,    0,   45,   63,    0,   83,    0,   80,
    0,  146,  147,    0,    0,    0,    0,   94,    0,    0,
   88,   90,  192,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  118,  119,  120,  121,    0,    0,
  117,  123,  116,    0,  197,  190,  191,    0,    0,    0,
  193,  204,  209,   50,   82,    0,    0,    0,    0,   77,
  145,   76,    0,   93,   98,    0,    0,    0,    0,  124,
  136,  137,  172,  126,   96,    0,  177,  178,   97,    0,
  176,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  112,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   95,    0,    0,    0,    0,    0,    0,  123,  123,  129,
    0,  125,    0,    0,    0,    0,    0,  175,    0,  123,
    0,    0,  123,  162,  157,  156,  100,  101,  111,  102,
    0,    0,    0,    0,    0,  113,  122,    0,  207,  208,
  206,  211,  210,   71,    0,  152,    0,  123,    0,    0,
    0,    0,    0,  139,    0,  138,    0,    0,    0,    0,
    0,    0,  184,    0,    0,    0,  201,    0,  123,    0,
  153,    0,  103,  104,  114,    0,    0,  202,  148,  151,
    0,  149,    0,  170,    0,    0,  167,    0,  133,    0,
    0,    0,  127,  128,  134,    0,    0,    0,  174,  183,
    0,    0,    0,    0,    0,    0,  109,  105,  107,    0,
    0,    0,    0,  144,  169,    0,  143,  166,  140,    0,
    0,  179,  180,    0,  181,    0,    0,    0,    0,  164,
  160,  115,    0,    0,    0,    0,  171,    0,  132,  130,
  185,    0,  123,  155,  154,    0,    0,    0,  203,  150,
  168,    0,    0,    0,  159,  106,  108,    0,  182,    0,
  131,    0,  161,
};
const short yydgoto[] = {                                       1,
    4,    5,    2,   57,   58,   45,  167,   93,   97,   98,
   95,  116,   42,   43,  254,  281,  179,  408,  180,  405,
  132,  232,  233,  354,  309,  289,  337,  312,  264,  301,
  202,  203,  384,  182,  183,  387,  204,  347,  385,  184,
  388,  265,  269,  270,  271,  363,  364,  191,   14,   25,
   49,   79,  108,   26,   48,   29,   32,   35,   38,   46,
   75,   76,   30,  161,  139,  162,   33,   36,   39,   96,
   47,   80,   81,  168,  151,  199,   82,  153,  130,  173,
  169,  109,  174,  154,  212,  256,  433,  434,  452,  412,
  442,  112,   84,  138,  185,  189,  192,  410,  343,  294,
};
const short yysindex[] = {                                      0,
    0, -278, -218,    0,    0, -232,  -84, -245, -243, -208,
 -198,    0, -166, -128, -134,  289, -140,  -95, -103,  -59,
  -50, -182,  -70,    0,  289,  -36,  -31,    2,   20,    0,
  -29, -256,    0,    8,   61,    0,   30,  108,    0,    0,
    0,    0, -292,    0, -242, -231,    0,  145,  -72,   49,
    0,    0,   37,    0,    0,   60,    0,  -29,    0,    0,
  125,    0,    0, -281,    0,    0,    0,    0, -162,    0,
    0,    0,  -30,    0,    0,    0,   77,   75,  127,  126,
    0,  142,    0,    0,  117,  148,    0,  152,    0,  167,
 -281,    0, -281,    0,    0,    0,    0, -117,  175,    0,
    0,    0,  -61,   31,  177,    0,  194,  186,    0,    0,
  208,  214,    0,    0,  173,    0,  202,  215,  239,    0,
    0,  218,  255,  267,  269,  271,  277,  278,    0,  318,
    0,    0,  127,    0,    0,  290,    0,    0,  281,  273,
  280,    0,    0,  282,    0,    0,    0,    0,    0,    0,
  256,  256,  -60,    0,    0,  -70,  -79, -228,  286,  317,
    0,    0,  259, -281, -281,    0, -305,  256,    0,  256,
 -191,  291,    0,  -69, -204,    0,   57,  268,    0,    0,
    0,    0,    0,  -79,  288,   82,    0,    0,  292,  133,
    0,  251,  265,  293,    0,    0, -281,    0,  294,    0,
 -233,    0,    0, -124,  296,  256,  291,    0, -186,  -34,
    0,    0,    0,  291,  191, -186,  -79,  196,  291,  291,
  291,  291,  291, -186,    0,    0,    0,    0,  270,  272,
    0,    0,    0,  275,    0,    0,    0,  262,  279,  283,
    0,    0,    0,    0,    0,  291,  291,  201,  284,    0,
    0,    0,  256,    0,    0, -160, -214, -186,  297,    0,
    0,    0,    0,    0,    0,  -32,    0,    0,    0,  210,
    0,  -79, -145, -169,  298,  163,   26,  300,  303,  304,
  261,    0,  264,  291,  291, -186,  299,  301,  105,  305,
  307,  309,  313,  315,  316,  -52, -142, -186,  310,  -54,
    0, -191,  291,  306,  291,  306,  295,    0,    0,    0,
 -186,    0,  291,  319,  291,  308,   -5,    0,  234,    0,
 -293,  320,    0,    0,    0,    0,    0,    0,    0,    0,
  323,  324,  325,  311,  312,    0,    0,  314,    0,    0,
    0,    0,    0,    0, -266,    0,  -92,    0,  321,  327,
  328,  329,  322,    0, -186,    0,  326,  110,  120,  330,
  196,  331,    0,  266,  196,  332,    0,  128,    0,  342,
    0,   58,    0,    0,    0,  333,  333,    0,    0,    0,
  143,    0, -191,    0,  274,  334,    0,  276,    0,  287,
 -186,  335,    0,    0,    0,  336,  341,  210,    0,    0,
  338,  -73,  153,  291,  343,  344,    0,    0,    0,  -79,
  196, -191,   22,    0,    0, -191,    0,    0,    0,  345,
  333,    0,    0,  161,    0,  339,  346,  348,  342,    0,
    0,    0,  291,  291, -272,  130,    0,  199,    0,    0,
    0, -186,    0,    0,    0,  349,  351,  352,    0,    0,
    0, -186,  353,  178,    0,    0,    0,  354,    0,  342,
    0,  355,    0,
};
const short yyrindex[] = {                                      0,
    0,  636,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  302,    0,    0,    0,    0,    0,
 -188,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  370,    0,
    0,    0,    0,    0,    0,    0,    0, -154,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  370,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  -28,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  356,    0,   32,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  358,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  -48,    0,    0,    0,    0,    0, -227,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,   89,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  118,    0,  166,    0,  166,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  359,
    0,    0,    0,    0,    0,  126,  126,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  226,    0,    0,    0,    0,    0,    0,    0,  359,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  359,
    0,    0,    0,
};
const short yygindex[] = {                                      0,
    0,    0,    0,  581,    0,    0,    0,    0,  -42,  551,
  -67,    0,  627,    0,    0,  452, -152, -349,    0, -372,
    0,    0,    0,  368,    0, -287, -184,    0,    0,    0,
 -203, -157,  337,    0, -179,  340, -248,    0,    0, -213,
    0,    0,    0,  285, -267,  347,    0,  483,    0,    0,
    0,  565,    0,  652,    0,    0,    0,    0,    0,  522,
    0,    0,  650,    0,    0,    0,  648,  646,  644,  615,
  -39,    0,    0, -143,    0,    0,    0,    0,    0,    0,
 -155,  552,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
};
#define YYTABLESIZE 728
const short yytable[] = {                                     181,
  251,  101,  318,  276,  235,  188,   72,  302,  170,  197,
   88,  176,  200,  248,  200,   67,  218,  369,   94,  205,
  358,  359,   88,   89,  263,  268,  181,  409,    3,  198,
  274,  275,  368,  279,  177,  372,  178,  449,    6,  286,
  247,  135,  135,  135,  135,  135,  248,   40,   41,  218,
  119,   59,   92,  135,  255,  121,  446,  267,  319,  181,
  381,  272,  253,  298,  303,  278,  280,  282,  282,  284,
  285,  440,   69,  310,    7,  121,   71,   31,  186,  135,
  135,  403,  135,  135,  135,  268,  135,  462,   10,  322,
   11,   70,  176,  295,  296,  249,  235,  200,  251,   12,
  304,  333,   44,  213,  187,  305,  135,  135,  135,   13,
  135,  321,  306,  322,  181,  201,  307,  267,  181,   33,
  257,  195,  196,  176,   17,  308,  360,  258,  329,   44,
  329,  331,  332,  303,  413,   72,  303,  257,  346,  235,
   15,   88,   89,   99,  258,   33,  300,  259,  260,  261,
  352,  262,  355,   32,  245,  454,  318,   40,   41,  176,
  361,  181,  365,  436,  259,  260,  261,  438,  262,  304,
  391,  100,  304,    8,  305,    9,  397,  305,   16,   32,
  391,  306,  201,  250,  306,  307,   88,   89,  307,  380,
  120,  176,   77,   27,  308,  320,  435,  308,  348,  206,
  207,  208,  209,  210,  176,   78,  420,  426,  396,  251,
  171,   28,  331,  268,  345,  379,   92,  428,  172,  247,
   92,   92,   92,   92,   92,  248,  428,  177,  218,  178,
   31,  176,  251,  277,  251,  123,  124,  125,  211,  268,
  258,  313,   79,  350,  351,  267,  303,   34,  216,  427,
   79,  430,  181,  370,  345,  235,   37,  453,  427,   92,
  259,  260,  261,   44,  262,  103,  314,  458,  313,  104,
   51,  267,  266,  303,  249,  216,   52,  181,   53,  258,
  447,  448,  304,   56,  219,  220,  221,  315,  223,  224,
  225,  226,  227,  228,  306,  229,  230,  316,  307,  259,
  260,  261,   78,  262,  303,  176,  216,  308,  231,  304,
   78,  219,  220,  221,  315,  223,  224,  225,  226,  227,
  228,  306,  229,  230,  316,  307,   28,   54,  201,  437,
  214,  126,  127,  128,  308,  231,  215,  216,  217,  218,
  304,   61,  219,  220,  221,  315,  223,  224,  225,  226,
  227,  228,  306,  229,  230,   83,  307,  173,  173,  173,
  173,  173,  216,   64,  257,  308,  231,   34,   62,  406,
   85,  258,   86,  219,  220,  221,  222,  223,  224,  225,
  226,  227,  228,  105,  229,  230,  142,  142,  142,  142,
  142,  259,  260,  261,  107,  262,  173,  231,  219,  220,
  221,  222,  223,  224,  225,  226,  227,  228,  106,  229,
  230,  257,  336,  176,   37,   65,  257,  393,  258,  238,
  239,  240,  231,  258,  113,  142,  257,  394,   88,   89,
   90,   91,  110,  258,  257,  402,  201,  450,  259,  260,
  261,  258,  262,  259,  260,  261,  176,  262,  111,  257,
  411,   73,   74,  259,  260,  261,  258,  262,   92,  257,
  115,  259,  260,  261,  429,  262,  258,  317,  441,  177,
  324,  178,  141,  141,  258,  117,  259,  260,  261,  141,
  262,  114,  176,  122,  257,   78,  259,  260,  261,  460,
  262,  258,  133,  134,  259,  260,  261,  273,  262,  141,
  141,  141,  277,  141,  258,  201,  451,  297,  140,  258,
  129,  259,  260,  261,  258,  262,  317,  176,   88,   89,
  136,  137,  142,  258,  259,  260,  261,  131,  262,  259,
  260,  261,  110,  262,  259,  260,  261,  141,  262,  110,
  177,  367,  178,  259,  260,  261,  143,  262,   92,   18,
   19,   20,   21,  144,   22,   23,  156,  190,  241,  110,
  110,  110,  145,  110,  157,  158,   67,  186,  328,   67,
  186,  330,  362,  399,  146,  159,  147,   24,  148,   67,
  383,  414,  386,  417,  149,  150,  152,  160,  163,  164,
  166,  165,  190,  193,  194,  236,  234,  186,  243,  237,
  291,  246,  244,  252,  287,  311,  288,  325,  323,  290,
  326,  327,  419,  334,  339,  335,  340,  292,  299,  338,
  341,  293,  342,  344,  349,  362,  404,  371,  353,  357,
  373,  374,  375,  383,  386,    1,  389,   73,   87,  395,
  392,  118,  366,  422,  376,  377,  401,  378,  423,  443,
  431,  432,  439,  444,  382,  445,  455,  390,  456,  457,
  459,  461,  463,   70,  407,  194,  165,  398,  421,   68,
  416,  425,  283,  356,  242,  135,   50,  175,   55,   60,
   63,   66,  424,  102,  155,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  400,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  415,    0,    0,    0,    0,    0,  418,
};
const short yycheck[] = {                                     157,
  204,   69,  270,  217,  184,  158,   46,  256,  152,  315,
  304,  284,  168,  280,  170,  308,  283,  311,   61,  172,
  308,  309,  304,  305,  209,  210,  184,  377,  307,  335,
  215,  216,  320,  218,  307,  323,  309,  310,  257,  224,
  274,  269,  270,  271,  272,  273,  280,  340,  341,  283,
   93,  308,  334,  281,  207,   98,  429,  210,  272,  217,
  348,  214,  206,  248,  279,  218,  219,  220,  221,  222,
  223,  421,  315,  258,  307,  118,  308,  334,  307,  307,
  308,  369,  310,  311,  312,  270,  314,  460,  334,  274,
  334,  334,  284,  246,  247,  329,  276,  253,  302,  308,
  315,  286,  334,  308,  333,  320,  334,  335,  336,  308,
  338,  281,  327,  298,  272,  307,  331,  270,  276,  308,
  307,  164,  165,  284,  259,  340,  311,  314,  281,  334,
  283,  284,  285,  279,  383,  175,  279,  307,  296,  319,
  307,  304,  305,  306,  314,  334,  307,  334,  335,  336,
  303,  338,  305,  308,  197,  443,  424,  340,  341,  284,
  313,  319,  315,  412,  334,  335,  336,  416,  338,  315,
  355,  334,  315,  258,  320,  260,  361,  320,  307,  334,
  365,  327,  307,  308,  327,  331,  304,  305,  331,  347,
  308,  284,  265,  334,  340,  341,  410,  340,  341,  269,
  270,  271,  272,  273,  284,  278,  391,  281,  361,  413,
  271,  307,  365,  398,  307,  308,  334,  402,  279,  274,
  269,  270,  271,  272,  273,  280,  411,  307,  283,  309,
  334,  284,  436,  307,  438,  297,  298,  299,  308,  424,
  314,  274,  271,  298,  299,  398,  279,  307,  281,  402,
  279,  404,  410,  321,  307,  435,  307,  442,  411,  308,
  334,  335,  336,  334,  338,  296,  299,  452,  274,  300,
  307,  424,  307,  279,  329,  281,  308,  435,  277,  314,
  433,  434,  315,  313,  317,  318,  319,  320,  321,  322,
  323,  324,  325,  326,  327,  328,  329,  330,  331,  334,
  335,  336,  271,  338,  279,  284,  281,  340,  341,  315,
  279,  317,  318,  319,  320,  321,  322,  323,  324,  325,
  326,  327,  328,  329,  330,  331,  307,  308,  307,  308,
  274,  301,  302,  303,  340,  341,  280,  281,  282,  283,
  315,  334,  317,  318,  319,  320,  321,  322,  323,  324,
  325,  326,  327,  328,  329,  307,  331,  269,  270,  271,
  272,  273,  281,  334,  307,  340,  341,  307,  308,  312,
  334,  314,  313,  317,  318,  319,  320,  321,  322,  323,
  324,  325,  326,  307,  328,  329,  269,  270,  271,  272,
  273,  334,  335,  336,  268,  338,  308,  341,  317,  318,
  319,  320,  321,  322,  323,  324,  325,  326,  334,  328,
  329,  307,  308,  284,  307,  308,  307,  308,  314,  287,
  288,  289,  341,  314,  308,  308,  307,  308,  304,  305,
  306,  307,  307,  314,  307,  308,  307,  308,  334,  335,
  336,  314,  338,  334,  335,  336,  284,  338,  307,  307,
  308,  307,  308,  334,  335,  336,  314,  338,  334,  307,
  309,  334,  335,  336,  312,  338,  314,  307,  308,  307,
  308,  309,  307,  308,  314,  309,  334,  335,  336,  314,
  338,  334,  284,  309,  307,  278,  334,  335,  336,  312,
  338,  314,  307,  308,  334,  335,  336,  307,  338,  334,
  335,  336,  307,  338,  314,  307,  308,  307,  336,  314,
  334,  334,  335,  336,  314,  338,  307,  284,  304,  305,
  307,  308,  308,  314,  334,  335,  336,  334,  338,  334,
  335,  336,  307,  338,  334,  335,  336,  336,  338,  314,
  307,  308,  309,  334,  335,  336,  308,  338,  334,  261,
  262,  263,  264,  336,  266,  267,  267,  307,  308,  334,
  335,  336,  308,  338,  275,  276,  265,  307,  308,  268,
  307,  308,  307,  308,  308,  286,  308,  289,  308,  278,
  307,  308,  307,  308,  308,  308,  269,  307,  316,  310,
  335,  310,  307,  277,  336,  308,  329,  307,  334,  308,
  339,  308,  310,  308,  335,  309,  335,  308,  311,  335,
  308,  308,  326,  315,  308,  315,  308,  339,  335,  315,
  308,  339,  308,  308,  315,  307,  285,  308,  323,  335,
  308,  308,  308,  307,  307,    0,  308,  268,   58,  310,
  315,   91,  335,  308,  334,  334,  315,  334,  308,  311,
  308,  308,  308,  308,  334,  308,  308,  336,  308,  308,
  308,  308,  308,  308,  332,  308,  308,  337,  334,   43,
  337,  334,  221,  306,  192,  111,   25,  156,   29,   32,
   35,   38,  398,   69,  133,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  364,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  385,   -1,   -1,   -1,   -1,   -1,  388,
};
#define YYFINAL 1
#ifndef YYDEBUG
#define YYDEBUG 0
#endif
#define YYMAXTOKEN 345
#if YYDEBUG
const char * const yyname[] = {
"end-of-file",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"DEFINE","DOMAIN","CDOMAIN",
"PROBLEM","REQUIREMENTS","TYPES","FUNCTIONS","PREDICATES","PREDICATE",
"EXTERNAL","OBJECTS","ACTION","PARAMETERS","PRECONDITION","EFFECT","COST",
"OBSERVATION","WHEN","INIT","GOAL","PACKAGE","RAMIFICATION","FORMULA","SET",
"IN","ONEOF","PRINT","PRINTSTATE","ASSERT","HOOKS","BELIEF","HEURISTIC","MODEL",
"PLANNING","CONFORMANT","ND_MDP","ND_POMDP","MDP","POMDP","DYNAMICS",
"DETERMINISTIC","NON_DETERMINISTIC","PROBABILISTIC","FEEDBACK","COMPLETE",
"PARTIAL","NULLF","INTEGER","BOOLEAN","ARRAY","LEFTPAR","RIGHTPAR","LEFTSQPAR",
"RIGHTSQPAR","LEFTCURLY","RIGHTCURLY","COLON","STAR","HYPHEN","COMMA","NOT",
"AND","OR","IF","IFF","EQUAL","LT","LE","GE","GT","PLUS","EXISTS","FORALL",
"VECTOR","SUM","INLINE","CERTAINTY","TOK_IDENT","TOK_VIDENT","TOK_INTEGER",
"TOK_FLOAT","TOK_BOOLEAN","TOK_STRING","TOK_FUNCTION","TOK_PREDICATE","BOGUS",
"BOGUS2","BOGUS3","BOGUS4",
};
const char * const yyrule[] = {
"$accept : start",
"start : code_chunks",
"code_chunks : code_chunks domain",
"code_chunks : code_chunks problem",
"code_chunks :",
"$$1 :",
"domain : LEFTPAR DEFINE LEFTPAR DOMAIN TOK_IDENT RIGHTPAR $$1 LEFTPAR global_list definition_list ramification_list action_list RIGHTPAR",
"global_list : global_list global LEFTPAR",
"global_list : global LEFTPAR",
"global : MODEL model_def RIGHTPAR",
"global : REQUIREMENTS requirement_list RIGHTPAR",
"global : TYPES type_list RIGHTPAR",
"global : FUNCTIONS function_list RIGHTPAR",
"global : PREDICATES predicate_list RIGHTPAR",
"global : EXTERNAL external_list RIGHTPAR",
"global : OBJECTS object_list RIGHTPAR",
"model_def : model_def dynamics_def",
"model_def : model_def feedback_def",
"model_def :",
"dynamics_def : LEFTPAR DYNAMICS DETERMINISTIC RIGHTPAR",
"dynamics_def : LEFTPAR DYNAMICS NON_DETERMINISTIC RIGHTPAR",
"dynamics_def : LEFTPAR DYNAMICS PROBABILISTIC RIGHTPAR",
"feedback_def : LEFTPAR FEEDBACK COMPLETE RIGHTPAR",
"feedback_def : LEFTPAR FEEDBACK PARTIAL RIGHTPAR",
"feedback_def : LEFTPAR FEEDBACK NULLF RIGHTPAR",
"requirement_list : requirement_list requirement",
"requirement_list : requirement",
"$$2 :",
"requirement : LEFTPAR PACKAGE TOK_IDENT RIGHTPAR $$2 rest_requirement",
"rest_requirement : package",
"type_list : type_list type_def",
"type_list : type_def",
"type_def : TOK_IDENT ancestor_list",
"type_def : TOK_IDENT",
"ancestor_list : ancestor_list ancestor",
"ancestor_list : ancestor",
"ancestor : COLON COLON TOK_IDENT",
"function_list : function_list function",
"function_list : function",
"function : LEFTPAR TOK_IDENT compound_type_decl simple_type_decl RIGHTPAR",
"predicate_list : predicate_list predicate",
"predicate_list : predicate",
"predicate : LEFTPAR TOK_IDENT simple_type_decl_list RIGHTPAR",
"compound_type_decl : simple_type_decl",
"compound_type_decl : LEFTPAR simple_type_decl_list RIGHTPAR",
"compound_type_decl : ARRAY LEFTSQPAR TOK_INTEGER RIGHTSQPAR simple_type_decl",
"simple_type_decl : TOK_IDENT",
"simple_type_decl : integer_type",
"simple_type_decl : boolean_type",
"integer_type : INTEGER range",
"range : LEFTSQPAR TOK_INTEGER COMMA TOK_INTEGER RIGHTSQPAR",
"boolean_type : BOOLEAN",
"simple_type_decl_list : simple_type_decl_list simple_type_decl",
"simple_type_decl_list : simple_type_decl",
"external_list : external_list external_decl",
"external_list : external_decl",
"external_decl : TOK_FUNCTION",
"external_decl : TOK_PREDICATE",
"object_list : object_list object_def",
"object_list : object_def",
"object_def : ident_list HYPHEN TOK_IDENT",
"object_def : ident_list HYPHEN integer_type",
"object_def : ident_list HYPHEN boolean_type",
"object_def : ident_list HYPHEN ARRAY LEFTSQPAR TOK_INTEGER RIGHTSQPAR simple_type_decl",
"ident_list : ident_list TOK_IDENT",
"ident_list : TOK_IDENT",
"definition_list : definition_list definition LEFTPAR",
"definition_list :",
"definition : logical_def",
"$$3 :",
"$$4 :",
"logical_def : PREDICATE LEFTPAR TOK_IDENT $$3 parameter_list $$4 RIGHTPAR formula RIGHTPAR",
"ramification_list : ramification_def LEFTPAR ramification_list",
"ramification_list :",
"$$5 :",
"ramification_def : RAMIFICATION TOK_IDENT $$5 optional_parameter ramification_def_rest",
"ramification_def_rest : FORMULA formula RIGHTPAR",
"ramification_def_rest : EFFECT atomic_effect_list RIGHTPAR",
"optional_parameter : PARAMETERS parameter_list",
"optional_parameter :",
"parameter_list : parameter_list parameter",
"parameter_list : parameter",
"parameter : variable_list HYPHEN simple_type_decl",
"variable_list : variable_list TOK_VIDENT",
"variable_list : TOK_VIDENT",
"action_list : action_list LEFTPAR action_def",
"action_list : action_def",
"$$6 :",
"action_def : ACTION action_name $$6 action_body_list RIGHTPAR",
"action_name : TOK_IDENT",
"action_body_list : action_body_list action_body",
"action_body_list :",
"action_body : PARAMETERS parameter_list",
"action_body : PRECONDITION precondition_formula",
"$$7 :",
"action_body : EFFECT $$7 effect_list",
"action_body : COST cost",
"action_body : OBSERVATION observation",
"precondition_formula : formula",
"formula : atomic_formula",
"formula : LEFTPAR NOT formula RIGHTPAR",
"formula : LEFTPAR AND formula_list RIGHTPAR",
"formula : LEFTPAR OR formula_list RIGHTPAR",
"formula : LEFTPAR IF formula formula RIGHTPAR",
"formula : LEFTPAR IFF formula formula RIGHTPAR",
"$$8 :",
"formula : LEFTPAR EXISTS TOK_VIDENT HYPHEN TOK_IDENT optional_inline $$8 formula RIGHTPAR",
"$$9 :",
"formula : LEFTPAR FORALL TOK_VIDENT HYPHEN TOK_IDENT optional_inline $$9 formula RIGHTPAR",
"optional_inline : INLINE",
"optional_inline :",
"formula_list : formula_list formula",
"formula_list : formula",
"atomic_formula : LEFTPAR predicate_symbol term_list RIGHTPAR",
"atomic_formula : LEFTPAR EQUAL term term RIGHTPAR",
"atomic_formula : LEFTPAR IN term LEFTCURLY term_list RIGHTCURLY RIGHTPAR",
"predicate_symbol : primitive_predicate_symbol",
"predicate_symbol : TOK_PREDICATE",
"primitive_predicate_symbol : LT",
"primitive_predicate_symbol : LE",
"primitive_predicate_symbol : GE",
"primitive_predicate_symbol : GT",
"term_list : term_list term",
"term_list :",
"term : TOK_VIDENT",
"term : TOK_IDENT optional_subscript",
"term : value",
"term : LEFTPAR TOK_FUNCTION term_list RIGHTPAR",
"term : LEFTPAR primitive_function_symbol term_list RIGHTPAR",
"term : STAR term",
"$$10 :",
"term : LEFTPAR SUM TOK_VIDENT HYPHEN TOK_IDENT optional_inline $$10 term RIGHTPAR",
"term : LEFTPAR IF formula term term RIGHTPAR",
"term : LEFTPAR FORMULA formula RIGHTPAR",
"optional_subscript : LEFTSQPAR term RIGHTSQPAR",
"optional_subscript :",
"value : TOK_INTEGER",
"value : TOK_BOOLEAN",
"primitive_function_symbol : PLUS optional_modulo",
"primitive_function_symbol : HYPHEN optional_modulo",
"optional_modulo : LT TOK_INTEGER GT",
"optional_modulo :",
"effect_list : atomic_effect_list",
"effect_list : LEFTPAR PROBABILISTIC probabilistic_effect_list RIGHTPAR",
"effect_list : LEFTPAR NON_DETERMINISTIC non_deterministic_effect_list RIGHTPAR",
"atomic_effect_list : atomic_effect_list atomic_effect",
"atomic_effect_list : atomic_effect",
"atomic_effect : basic_effect",
"atomic_effect : LEFTPAR WHEN formula basic_effect_list RIGHTPAR",
"$$11 :",
"atomic_effect : LEFTPAR FORALL TOK_VIDENT HYPHEN TOK_IDENT $$11 atomic_effect_list RIGHTPAR",
"basic_effect_list : basic_effect_list basic_effect",
"basic_effect_list : basic_effect",
"basic_effect : LEFTPAR SET term term RIGHTPAR",
"basic_effect : LEFTPAR SET LEFTPAR TOK_PREDICATE term_list RIGHTPAR term RIGHTPAR",
"basic_effect : LEFTPAR SET LEFTPAR TOK_PREDICATE term_list RIGHTPAR formula RIGHTPAR",
"basic_effect : LEFTPAR PRINT term RIGHTPAR",
"basic_effect : LEFTPAR PRINT formula RIGHTPAR",
"basic_effect : PRINTSTATE",
"init_set_effect : LEFTPAR SET term IN LEFTCURLY term_list RIGHTCURLY optional_assert RIGHTPAR",
"init_set_effect : LEFTPAR SET term IN integer_type optional_assert RIGHTPAR",
"init_set_effect : LEFTPAR SET LEFTPAR TOK_PREDICATE term_list RIGHTPAR IN LEFTCURLY term_list RIGHTCURLY optional_assert RIGHTPAR",
"init_set_effect : LEFTPAR ONEOF atomic_init_effect_list RIGHTPAR",
"init_set_effect : formula",
"optional_assert : ASSERT formula",
"optional_assert :",
"probabilistic_effect_list : probabilistic_effect_list probabilistic_effect",
"probabilistic_effect_list : probabilistic_effect",
"probabilistic_effect : LEFTPAR TOK_FLOAT atomic_effect_list RIGHTPAR",
"non_deterministic_effect_list : non_deterministic_effect_list non_deterministic_effect",
"non_deterministic_effect_list : non_deterministic_effect",
"non_deterministic_effect : LEFTPAR atomic_effect_list RIGHTPAR",
"cost : term",
"observation : observation_list",
"observation : LEFTPAR PROBABILISTIC probabilistic_observation_list RIGHTPAR",
"observation_list : observation_list atomic_observation",
"observation_list : atomic_observation",
"atomic_observation : formula",
"atomic_observation : term",
"atomic_observation : LEFTPAR WHEN formula formula RIGHTPAR",
"atomic_observation : LEFTPAR WHEN formula term RIGHTPAR",
"$$12 :",
"atomic_observation : LEFTPAR VECTOR TOK_VIDENT HYPHEN TOK_IDENT $$12 term RIGHTPAR",
"probabilistic_observation_list : probabilistic_observation_list probabilistic_observation",
"probabilistic_observation_list : probabilistic_observation",
"probabilistic_observation : LEFTPAR TOK_FLOAT observation_list RIGHTPAR",
"$$13 :",
"problem : LEFTPAR DEFINE LEFTPAR PROBLEM TOK_IDENT RIGHTPAR LEFTPAR CDOMAIN TOK_IDENT RIGHTPAR $$13 problem_def_list RIGHTPAR",
"problem_def_list : problem_def_list problem_def",
"problem_def_list :",
"problem_def : LEFTPAR INIT init_def RIGHTPAR",
"problem_def : LEFTPAR GOAL goal_def RIGHTPAR",
"problem_def : LEFTPAR OBJECTS object_list RIGHTPAR",
"problem_def : LEFTPAR HOOKS hook_list RIGHTPAR",
"init_def : atomic_init_effect_list",
"goal_def : formula",
"goal_def : CERTAINTY",
"atomic_init_effect_list : atomic_init_effect_list atomic_init_effect",
"atomic_init_effect_list : atomic_init_effect",
"atomic_init_effect : basic_effect",
"atomic_init_effect : init_set_effect",
"atomic_init_effect : LEFTPAR WHEN formula atomic_init_effect_list RIGHTPAR",
"$$14 :",
"atomic_init_effect : LEFTSQPAR FORALL TOK_VIDENT HYPHEN TOK_IDENT $$14 atomic_init_effect_list RIGHTSQPAR",
"hook_list : hook_list hook",
"hook_list : hook",
"hook : LEFTPAR MODEL TOK_STRING RIGHTPAR",
"hook : LEFTPAR BELIEF TOK_STRING RIGHTPAR",
"hook : LEFTPAR HEURISTIC TOK_STRING RIGHTPAR",
"$$15 :",
"package : LEFTPAR PACKAGE TOK_IDENT $$15 rest_package",
"rest_package : RIGHTPAR",
};
#endif
#if YYDEBUG
#include <stdio.h>
#endif
#ifdef YYSTACKSIZE
#undef YYMAXDEPTH
#define YYMAXDEPTH YYSTACKSIZE
#else
#ifdef YYMAXDEPTH
#define YYSTACKSIZE YYMAXDEPTH
#else
#define YYSTACKSIZE 10000
#define YYMAXDEPTH 10000
#endif
#endif
#define YYINITSTACKSIZE 200
int yydebug;
int yynerrs;
int yyerrflag;
int yychar;
short *yyssp;
YYSTYPE *yyvsp;
YYSTYPE yyval;
YYSTYPE yylval;
short *yyss;
short *yysslim;
YYSTYPE *yyvs;
int yystacksize;
#line 1413 "daeParser.y"


//
// Function used by YACC to print syntax errors.
//
void
yyerror( const char *s )
{
  extern int lineno;
#ifdef sparc
  extern char yytext[];      // needed in my g++/sparc
#endif
#ifdef linux
  extern char *yytext;       // needed in my g++/linux
#endif

  insertFmtError( "%s:%d: %s before \"%s\"\n", basename( yyfile ), lineno, s, yytext );
}


//
// Initialization of parsing process. We allocate primitive types and 
// predicates, create basic hash tables, and initialize the package 
// processing facility. This function must be called just once.
//
void
initParser( void )
{
  list<const typeClass*> *domain;

  // declared objects
  declaredDomain = NULL;
  declaredProblem = NULL;
  declaredAction = NULL;
  variableEnvironment.clear();
  parameterList.clear();

  actionSet.clear();
  ramificationSet.clear();
  internalPredicateSet.clear();

  // constant types
  theBooleanType = new booleanTypeClass();
  theGenericIntegerType = new genericIntegerTypeClass();

  // hashs
  domainHash.clear();
  problemSet.clear();
  globalFunctionHash.clear();
  globalPredicateHash.clear();
  globalNamedTypeHash.clear();

  // primitive functions
  domain = new list<const typeClass*>;
  domain->push_back( new genericIntegerTypeClass );
  domain->push_back( new genericIntegerTypeClass );
  plusFunctionDomain = new typeListClass( domain );
  plusFunctionRange = new genericIntegerTypeClass;
  minusFunctionDomain = (typeListClass*)plusFunctionDomain->clone();
  minusFunctionRange = new genericIntegerTypeClass;

  // primitive predicates
  theLessThanPredicate = new predicateClass( strdup("_lessThan"), plusFunctionDomain->clone(), 
					     predicateClass::SIMPLE_DOMAIN, 1 );
  theLessEqualPredicate = new predicateClass( strdup("_lessEqual"), plusFunctionDomain->clone(), 
					      predicateClass::SIMPLE_DOMAIN, 1 );
  theGreaterEqualPredicate = new predicateClass( strdup("_greaterEqual"), plusFunctionDomain->clone(),
						 predicateClass::SIMPLE_DOMAIN, 1 );
  theGreaterThanPredicate = new predicateClass( strdup("_greaterThan"), plusFunctionDomain->clone(),
						predicateClass::SIMPLE_DOMAIN, 1 );
  globalPredicateHash.insert( make_pair( (const char*)"_lessThan", theLessThanPredicate ) );
  globalPredicateHash.insert( make_pair( (const char*)"_lessEqual", theLessEqualPredicate ) );
  globalPredicateHash.insert( make_pair( (const char*)"_greaterEqual", theGreaterEqualPredicate ) );
  globalPredicateHash.insert( make_pair( (const char*)"_greaterThan", theGreaterThanPredicate ) );

  domain = new list<const typeClass*>;
  domain->push_back( new universalTypeClass() );
  domain->push_back( new universalTypeClass() );
  theEqualPredicate = new predicateClass( strdup("="), new typeListClass( domain ), 
					  predicateClass::SIMPLE_DOMAIN );
  globalPredicateHash.insert( make_pair( (const char*)"=", theEqualPredicate ) );

  // primitive types
  globalNamedTypeHash.insert( make_pair( (const char*)":integer", (const namedTypeClass*)NULL ) );
  globalNamedTypeHash.insert( make_pair( (const char*)":boolean", (const namedTypeClass*)NULL ) );

  // packages
  initPackages();
}


static void
startDomain( const char *ident )
{
  map<const char*,const functionClass*,ltcasestr>::const_iterator it1;
  map<const char*,const predicateClass*,ltcasestr>::const_iterator it2;
  map<const char*,const namedTypeClass*,ltcasestr>::const_iterator it3;

  if( domainHash.find( ident ) != domainHash.end() )
    {
      error( nameClash, ident );
      return;
    }

  // allocate domain
  declaredDomain = new domainClass( strdup( yyfile ), ident );

  // default hashes
  for( it1 = globalFunctionHash.begin(); it1 != globalFunctionHash.end(); ++it1 )
    declaredDomain->functionHash.insert( make_pair( (*it1).first, new functionClass( *(*it1).second ) ) );

  for( it2 = globalPredicateHash.begin(); it2 != globalPredicateHash.end(); ++it2 )
    declaredDomain->predicateHash.insert( make_pair( (*it2).first, new predicateClass( *(*it2).second ) ) );

  for( it3 = globalNamedTypeHash.begin(); it3 != globalNamedTypeHash.end(); ++it3 )
    if( (*it3).second )
      declaredDomain->namedTypeHash.insert( make_pair((*it3).first,new namedTypeClass(*(*it3).second)) );
    else
      declaredDomain->namedTypeHash.insert( make_pair( (*it3).first, (const namedTypeClass*)NULL ) );

  // setup global hashes
  namedTypeHash = &declaredDomain->namedTypeHash;
  objectHashByType = &declaredDomain->objectHashByType;
  objectHashByName = &declaredDomain->objectHashByName;
  ramificationList = &declaredDomain->ramificationList;
  internalPredicateList = &declaredDomain->internalPredicateList;
  actionList = &declaredDomain->actionList;
  functionHash = &declaredDomain->functionHash;
  predicateHash = &declaredDomain->predicateHash;
  arrayHash = &declaredDomain->arrayHash;
}


static const codeChunkClass *
finishDomain( void )
{
  // save domain in global hash table
  domainHash.insert( make_pair( declaredDomain->ident, declaredDomain ) );

  actionSet.clear();
  ramificationSet.clear();
  internalPredicateSet.clear();

  // cleanning
  namedTypeHash = NULL;
  objectHashByType = NULL;
  objectHashByName = NULL;
  ramificationList = NULL;
  internalPredicateList = NULL;
  actionList = NULL;
  functionHash = NULL;
  predicateHash = NULL;
  arrayHash = NULL;
  domainClass *rv = declaredDomain;
  declaredDomain = NULL;
  return( rv );
}


void
cleanDomains( void )
{
  map<const char*,const domainClass*,ltcasestr>::const_iterator it;

  map<const char*,const functionClass*,ltcasestr>::const_iterator it1;
  map<const char*,const predicateClass*,ltcasestr>::const_iterator it2;
  map<const char*,const namedTypeClass*,ltcasestr>::const_iterator it3;
  map<const char*,list<const char*>*,ltcasestr>::const_iterator it4;
  map<const char*,const typeClass*,ltcasestr>::const_iterator it5;
  map<const char*,const arrayClass*,ltcasestr>::const_iterator it6;
  list<const ramificationClass*>::const_iterator it7;
  list<const internalFunctionClass*>::const_iterator it8;
  list<const internalPredicateClass*>::const_iterator it9;
  list<const actionClass*>::const_iterator it10;

  for( it = domainHash.begin(); it != domainHash.end(); ++it )
    {
      //      fprintf( stderr, "free of [%p]%s\n", ((*it).second->ident), ((*it).second->ident) );
      //      fprintf( stderr, "free of [%p]%s\n", ((*it).second->fileName), ((*it).second->fileName) );
      free( (void*)((*it).second->ident) );
      free( (void*)((*it).second->fileName) );

      for( it1 = (*it).second->functionHash.begin(); it1 != (*it).second->functionHash.end(); ++it1 );

      // clean predicateHash
      for( it2 = (*it).second->predicateHash.begin(); it2 != (*it).second->predicateHash.end(); ++it2 )
	delete (*it2).second;

      // clean namedTypeHash
      for( it3 = (*it).second->namedTypeHash.begin(); it3 != (*it).second->namedTypeHash.end(); ++it3 )
	delete (*it3).second;

      // clean objectHashByType
      for( it4 = (*it).second->objectHashByType.begin(); it4 != (*it).second->objectHashByType.end(); ++it4 )
	{
	  list<const char*>::const_iterator it;
	  //	  fprintf( stderr, "free of [%p]%s\n", (*it4).first, (*it4).first );
	  free( (void*)((*it4).first) );
	  for( it = (*it4).second->begin(); it != (*it4).second->end(); ++it )
	    {
	      //	      fprintf( stderr, "free of [%p]%s\n", (*it), (*it) );
	      free( (void*)(*it) );
	    }
	  delete (*it4).second;
	}

      // clean objectHashByName
      for( it5 = (*it).second->objectHashByName.begin(); it5 != (*it).second->objectHashByName.end(); ++it5 )
	{
	  //	  fprintf( stderr, "free of [%p]%s\n", (*it5).first, (*it5).first );
	  free( (void*)(*it5).first );
	  delete (*it5).second;
	}

      // clean arrayHash
      for( it6 = (*it).second->arrayHash.begin(); it6 != (*it).second->arrayHash.end(); ++it6 )
	{
	  //	  fprintf( stderr, "free of [%p]%s\n", (*it6).first, (*it6).first );
	  free( (void*)(*it6).first );
	  delete (*it6).second;
	}

      // clean ramification
      for( it7 = (*it).second->ramificationList.begin(); it7 != (*it).second->ramificationList.end(); ++it7 )
	delete (*it7);

      // clean internalFunctions
      for( it8 = (*it).second->internalFunctionList.begin(); it8 != (*it).second->internalFunctionList.end(); ++it8 )
	delete (*it8);

      // clean internalPredicates
      for( it9 = (*it).second->internalPredicateList.begin(); it9 != (*it).second->internalPredicateList.end(); ++it9 )
	delete (*it9);

      // clean actions
      for( it10 = (*it).second->actionList.begin(); it10 != (*it).second->actionList.end(); ++it10 )
	delete (*it10);
    }
}


static void
declareModel( void )
{
  assert( declaredDomain != NULL );
  switch( declaredDomain->feedback )
    {
    case COMPLETE:
      switch( declaredDomain->dynamics )
	{
	case DETERMINISTIC:
	  //declaredDomain->model = PLANNING;
	  declaredDomain->model = MDP;
	  break;
	case NON_DETERMINISTIC:
	  declaredDomain->model = ND_MDP;
	  break;
	case PROBABILISTIC:
	  declaredDomain->model = MDP;
	  break;
	}
      break;
    case PARTIAL:
      switch( declaredDomain->dynamics )
	{
	case DETERMINISTIC:
	case NON_DETERMINISTIC:
	  declaredDomain->model = ND_POMDP;
	  break;
	case PROBABILISTIC:
	  declaredDomain->model = POMDP;
	  break;
	}
      break;
    case NULLF:
      switch( declaredDomain->dynamics )
	{
	case DETERMINISTIC:
	case NON_DETERMINISTIC:
	  declaredDomain->model = CONFORMANT;
	  break;
	case PROBABILISTIC:
	  declaredDomain->model = POMDP;
	  break;
	}
      break;
    }
}


static void
declareDynamics( int dynamics )
{
  assert( declaredDomain != NULL );
  if( (dynamics != PROBABILISTIC) && (dynamics != NON_DETERMINISTIC) && (dynamics != DETERMINISTIC) )
    error( undefinedModel );
  else
    declaredDomain->dynamics = dynamics;
}


static void
declareFeedback( int feedback )
{
  assert( declaredDomain != NULL );
  if( (feedback != COMPLETE) && (feedback != PARTIAL) && (feedback != NULLF) )
    error( undefinedModel );
  else
    declaredDomain->feedback = feedback;
}


static void
declareExternal( const list<const char*> *eList )
{
  list<const char*>::const_iterator it;

  assert( declaredDomain != NULL );
  for( it = eList->begin(); it != eList->end(); ++it )
    {
      map<const char*,const functionClass*,ltcasestr>::iterator it1;
      map<const char*,const predicateClass*,ltcasestr>::iterator it2;

      it1 = declaredDomain->functionHash.find( (*it) );
      it2 = declaredDomain->predicateHash.find( (*it) );

      if( (it1 == declaredDomain->functionHash.end()) && (it2 == declaredDomain->predicateHash.end()) )
	error( undefinedFunPred, (*it) );

      if( it1 != declaredDomain->functionHash.end() )
	{
	  if( (*it1).second->external == 0 )
	    {
	      ((functionClass*)(*it1).second)->external = 1;
	      --declaredDomain->numFunctions[(*it1).second->numArguments];
	    }
	}

      if( it2 != declaredDomain->predicateHash.end() )
	{
	  if( (*it2).second->external == 0 )
	    {
	      ((predicateClass*)(*it2).second)->external = 1;
	      --declaredDomain->numPredicates[(*it2).second->numArguments];
	    }
	}

      free( (void*)(*it) );
    }
  delete eList;
}


static void
declareNamedType( const char *ident, const list<const char*> *aList )
{
  assert( namedTypeHash != NULL );
  if( namedTypeHash->find( ident ) != namedTypeHash->end() )
    {
      error( nameClash, ident );
    }
  else
    {
      namedTypeClass *type = new namedTypeClass( ident, aList );
      namedTypeHash->insert( make_pair( ident, type ) );
    }
}


static void
declareFunction( const char *ident, const typeClass *domain, const typeClass *range )
{
  functionClass *function;

  assert( functionHash != NULL );
  assert( declaredDomain != NULL );

  if( functionHash->find( ident ) != functionHash->end() )
    {
      error( nameClash, ident );
    }
  else
    {
      function = new functionClass( ident, domain, range, domainType );
      functionHash->insert( make_pair( ident, function ) );
      ++declaredDomain->numFunctions[function->numArguments];
    }
}


static void
declarePredicate( const char *ident, const typeClass *domain )
{
  predicateClass *predicate;

  assert( predicateHash != NULL );
  assert( declaredDomain != NULL );

  if( predicateHash->find( ident ) != predicateHash->end() )
    {
      error( nameClash, ident );
    }
  else
    {
      predicate = new predicateClass( ident, domain, domainType );
      predicateHash->insert( make_pair( ident, predicate ) );
      ++declaredDomain->numPredicates[predicate->numArguments];
    }
}


static void
declareSimpleObjects( const char *ident, const list<const char*> *oList )
{
  typeClass *type;
  list<const char*>::const_iterator it1;
  map<const char*,list<const char*>*,ltcasestr>::const_iterator it;

  assert( objectHashByType != NULL );
  assert( objectHashByName != NULL );

  // classified insertion of objects into domain objects
  it = objectHashByType->find( ident );
  if( it == objectHashByType->end() )
    objectHashByType->insert( make_pair( strdup( ident ), new list<const char*>(*oList) ) );
  else
    {
      for( it1 = oList->begin(); it1 != oList->end(); ++it1 )
	(*it).second->push_back( *it1 );
    }

  // insert objects into hashByName
  type = new simpleTypeClass( ident );
  for( it1 = oList->begin(); it1 != oList->end(); ++it1 )
    {
      if( objectHashByName->find( (*it1) ) != objectHashByName->end() )
	error( nameClash, (*it1) );
      else
	objectHashByName->insert( make_pair( strdup( *it1 ), type->clone() ) );
    }

  // clean 
  delete type;
  delete oList;
}


static void
declareIntegerObjects( const char *min, const char *max, const list<const char*> *oList )
{
  typeClass *type;
  list<const char*>::const_iterator it1;
  map<const char*,list<const char*>*,ltcasestr>::const_iterator it;

  assert( objectHashByType != NULL );
  assert( objectHashByName != NULL );

  // classified insertion of objects into domain objects
  it = objectHashByType->find( (const char*)":integer" );
  if( it == objectHashByType->end() )
    objectHashByType->insert( make_pair( strdup(":integer"), new list<const char*>(*oList) ) );
  else
    {
      for( it1 = oList->begin(); it1 != oList->end(); ++it1 )
	(*it).second->push_back( *it1 );
    }

  // insert objects into hashByName
  type = new integerTypeClass( min, max );
  for( it1 = oList->begin(); it1 != oList->end(); ++it1 )
    {
      if( objectHashByName->find( (*it1) ) != objectHashByName->end() )
	error( nameClash, (*it1) );
      else
	objectHashByName->insert( make_pair( strdup( *it1 ), type->clone() ) );
    }

  // clean 
  delete type;
  delete oList;
}


static void
declareBooleanObjects( const list<const char*> *oList )
{
  list<const char*>::const_iterator it1;
  map<const char*,list<const char*>*,ltcasestr>::const_iterator it;

  assert( objectHashByType != NULL );
  assert( objectHashByName != NULL );

  // classified insertion of objects into domain objects
  it = objectHashByType->find( (const char*)":boolean" );
  if( it == objectHashByType->end() )
    objectHashByType->insert( make_pair( strdup(":boolean"), new list<const char*>(*oList) ) );
  else
    {
      for( it1 = oList->begin(); it1 != oList->end(); ++it1 )
	(*it).second->push_back( *it1 );
    }

  // insert objects into hashByName
  for( it1 = oList->begin(); it1 != oList->end(); ++it1 )
    {
      if( objectHashByName->find( (*it1) ) != objectHashByName->end() )
	error( nameClash, (*it1) );
      else
	objectHashByName->insert( make_pair( strdup( *it1 ), theBooleanType->clone() ) );
    }

  // clean 
  delete oList;
}


static void
declareArrayObjects( const char *size, const typeClass *base, const list<const char*> *oList )
{
  int dim;
  typeClass *type;
  arrayClass *array;
  map<const char*,list<const char*>*,ltcasestr>::const_iterator it;
  list<const char*>::const_iterator it1;
  map<const char*,const arrayClass*,ltcasestr>::const_iterator it2;

  assert( arrayHash != NULL );
  assert( declaredDomain != NULL );
  assert( objectHashByType != NULL );
  assert( objectHashByName != NULL );

  // classified insertion of objects into domain objects
  it = objectHashByType->find( (const char*)":array" );
  if( it == objectHashByType->end() )
    objectHashByType->insert( make_pair( strdup(":array"), new list<const char*>(*oList) ) );
  else
    {
      for( it1 = oList->begin(); it1 != oList->end(); ++it1 )
	(*it).second->push_back( *it1 );
    }

  // insert objects into hashByName
  type = new arrayTypeClass( size, base );
  for( it1 = oList->begin(); it1 != oList->end(); ++it1 )
    {
      if( objectHashByName->find( (*it1) ) != objectHashByName->end() )
	error( nameClash, (*it1) );
      else
	objectHashByName->insert( make_pair( strdup( *it1 ), type->clone() ) );
    }

  // declare and insert new array into arrayHash
  for( it1 = oList->begin(); it1 != oList->end(); ++it1 )
    {
      it2 = arrayHash->find( (*it1) );
      if( it2 == arrayHash->end() )
	{
	  array = new arrayClass( strdup( (*it1) ), strdup(size), base->clone() );
	  arrayHash->insert( make_pair( strdup( (*it1) ), array ) );
	  if( declaredProblem != NULL )
	    declaredProblem->numArrays++;
	  else
	    declaredDomain->numArrays++;
	}
    }

  // update maxArraySize
  dim = atoi( size );
  if( declaredProblem != NULL )
    declaredProblem->maxArraySize = (declaredProblem->maxArraySize>dim?declaredProblem->maxArraySize:dim);
  else
    declaredDomain->maxArraySize = (declaredDomain->maxArraySize>dim?declaredDomain->maxArraySize:dim);

  // clean 
  delete type;
  delete oList;
}


static void
declareVariable( const char *ident, const typeClass *type )
{
  if( variableEnvironment.front()->find( ident ) == variableEnvironment.front()->end() )
    variableEnvironment.front()->insert( make_pair( ident, type ) );
  else
    error( nameClash, ident );
}


static void
startRamification( const char *ident )
{
  currentRamification = ident;
  if( ramificationSet.find( ident ) != ramificationSet.end() )
    error( nameClash, ident );
}


static void
declareRamification( const formulaClass *formula )
{
  ramificationClass *ram = new formulaRamificationClass( currentRamification, parameterList, formula );
  ramificationSet.insert( currentRamification );
  ramificationList->push_back( ram );
  cleanEnvironment( variableEnvironment.front() );
  variableEnvironment.pop_front();
  parameterList.clear();
  currentRamification = NULL;
}


static void
declareRamification( const effectClass *effect )
{
  ramificationClass *ram = new effectRamificationClass( currentRamification, parameterList, effect );
  ramificationSet.insert( currentRamification );
  ramificationList->push_back( ram );
  cleanEnvironment( variableEnvironment.front() );
  variableEnvironment.pop_front();
  parameterList.clear();
  currentRamification = NULL;

  ram->dirtyPredicates.insert( ramDirtyPredicates.begin(), ramDirtyPredicates.end() );
  ramDirtyPredicates.clear();
  ram->dirtyFunctions.insert( ramDirtyFunctions.begin(), ramDirtyFunctions.end() );
  ramDirtyFunctions.clear();
}


static void
startLogicalDef( const char *ident )
{
  if( internalPredicateSet.find( ident ) != internalPredicateSet.end() )
    error( nameClash, ident );
  else
    {
      list<pair<const char*,const typeClass*> >::const_iterator it;
      list<const typeClass*> *domain = new list<const typeClass*>;

      // create the type for the domain of the new predicate
      for( it = parameterList.begin(); it != parameterList.end(); ++it )
	domain->push_back( (*it).second->clone() );

      // create the entry in the corresponding hash table
      predicateClass *predicate = new predicateClass( strdup(ident), new typeListClass( domain ), 
						      predicateClass::LIST_DOMAIN );
      predicate->internal = 1;
      predicateHash->insert( make_pair( ident, predicate ) );
    }
}


static void
declareLogicalDef( const char *ident, const formulaClass *formula )
{
  internalPredicateClass *pred = new internalPredicateClass( ident, parameterList, formula );
  internalPredicateSet.insert( ident );
  internalPredicateList->push_back( pred );
  cleanEnvironment( variableEnvironment.front() );
  variableEnvironment.pop_front();
  parameterList.clear();
}


static void
startAction( const char *ident )
{
  actionClass *action;

  if( actionSet.find( ident ) != actionSet.end() )
    error( nameClash, ident );
  else
    {
      assert( actionList != NULL );
      action = new actionClass( ident );
      actionList->push_back( action );
      declaredAction = action;
    }
}


static void
finishAction( void )
{
  //xxxx
  declaredAction->parameters.insert( declaredAction->parameters.end(), 
				     parameterList.begin(), parameterList.end() );
  cleanEnvironment( variableEnvironment.front() );
  variableEnvironment.pop_front();
  assert( variableEnvironment.empty() );
  parameterList.clear();
  declaredAction = NULL;
}


static void
declareActionPrecondition( const formulaClass *precondition )
{
  assert( declaredAction != NULL );
  declaredAction->precondition = precondition;
}


static void
declareActionEffect( const effectClass *effect )
{
  assert( declaredAction != NULL );
  declaredAction->effect = effect;
}


static void
declareActionCost( const costClass *cost )
{
  assert( declaredAction != NULL );
  declaredAction->cost = cost;
}


static void
declareActionObservation( const observationClass *observation )
{
  assert( declaredDomain != NULL );
  assert( declaredAction != NULL );
  declaredAction->observation = observation;
}


static termClass *
fetchVariableTerm( const char *ident, termClass *subscript )
{
  termClass *result = NULL;
  list<map<const char*,const typeClass*,ltcasestr>*>::const_iterator it;
  map<const char*,const typeClass*,ltcasestr>::const_iterator it2;

  // in this implementation subscript is ALWAYS NULL
  assert( subscript == NULL );

  // fetch variable
  for( it = variableEnvironment.begin(); it != variableEnvironment.end(); ++it )
    {
      it2 = (*it)->find( ident );
      if( it2 != (*it)->end() )
	{
	  result = new variableTermClass( ident, (*it2).second->clone() );
	  break;
	}
    }

  // check for error
  if( it == variableEnvironment.end() )
    {
      error( undefinedVariable, ident );
      result = new universalTermClass();
    }

  // return value
  return( result );
}


static termClass *
fetchObjectTerm( const char *ident, const termClass *subscript )
{
  termClass *rv;
  list<const char*>::const_iterator it;
  map<const char*,const typeClass*,ltcasestr>::const_iterator it1;
  map<const char*,list<const char*>*,ltcasestr>::const_iterator it2;

  assert( objectHashByName != NULL );

  // get object type
  it1 = objectHashByName->find( ident );
  if( it1 == objectHashByName->end() )
    {
      error( undefinedObject, ident );
      rv = new universalTermClass();
      delete subscript;
    }
  else
    {
      // check if it is an array
      it2 = objectHashByType->find( (const char*)":array" );
      if( it2 != objectHashByType->end() )
	for( it = (*it2).second->begin(); it != (*it2).second->end(); ++it )
	  if( !strcmp( ident, (*it) ) )
	    break;

      // check for invalid susbcript
      if( (it2 == objectHashByType->end()) || (it == (*it2).second->end()) )
	{
	  if( subscript != NULL )
	    {
	      error( invalidSubscript, ident );
	      rv = new universalTermClass();
	    }
	  else
	    {
	      // simple object 
	      rv = new objectTermClass( ident, (*it1).second->clone() );
	    }
	  delete subscript;
	}
      else
	{
	  // we have an array either without subscript or with subscript
	  rv = new arrayTermClass( ident, (const arrayTypeClass*)(*it1).second->clone(), subscript );
	}
    }

  // return value
  return( rv );
}


static bool
checkProbabilities( const probabilisticEffectClass *effect )
{
  list<pair<const char*,const effectClass*>*>::const_iterator it;

  float prob = 0.0;
  for( it = effect->effectList->begin(); it != effect->effectList->end(); ++it )
    prob += atof( (*it)->first );

#if linux
  prob = (float)((unsigned)(1000.0*prob+0.5))/1000.0;
#endif

  if( prob > 1.0 ) error( probabilitySum );
  if( prob != 1.0 ) warning( probabilitySumWarning, prob );
  return( prob == 1.0 );
}


static void
checkProbabilities( const list<pair<const char*,const observationListClass*>*> *obsList )
{
  list<pair<const char*,const observationListClass*>*>::const_iterator it;

  float prob = 0.0;
  for( it = obsList->begin(); it != obsList->end(); ++it )
    prob += atof( (*it)->first );
  if( prob != 1.0 ) error( probabilitySum );
}


static void
startProblem( const char *problemIdent, const char *domainIdent )
{
  map<const char*,const domainClass*,ltcasestr>::const_iterator it;

  it = domainHash.find( domainIdent );
  if( it == domainHash.end() )
    {
      // this is a serious error, parsing can't continue.
      // Report it, printErrors on stderr, and exit.
      // this should be improved.
      error( undefinedDomain, domainIdent );
      printErrors( stderr );
      exit( -1 );
    }
  free( (void*)domainIdent );

  if( problemSet.find( problemIdent ) != problemSet.end() )
    error( nameClash, problemIdent );

  // allocate problem
  declaredProblem = new problemClass( strdup( yyfile ), problemIdent, (*it).second );

  // setup global hashes
  namedTypeHash = &declaredProblem->namedTypeHash;
  objectHashByType = &declaredProblem->objectHashByType;
  objectHashByName = &declaredProblem->objectHashByName;
  ramificationList = (list<const ramificationClass*>*)&declaredProblem->domain->ramificationList;
  internalPredicateList = (list<const internalPredicateClass*>*)&declaredProblem->domain->internalPredicateList;
  actionList = (list<const actionClass*>*)&declaredProblem->domain->actionList;
  functionHash = (map<const char*,const functionClass*,ltcasestr>*)&declaredProblem->domain->functionHash;
  predicateHash = (map<const char*,const predicateClass*,ltcasestr>*)&declaredProblem->domain->predicateHash;
  arrayHash = &declaredProblem->arrayHash;
}


static const codeChunkClass *
finishProblem( void )
{
  // save domain in global hash table
  problemSet.insert( make_pair( strdup( declaredProblem->ident ), 
				strdup( declaredProblem->domain->ident ) ) );

  actionSet.clear();
  ramificationSet.clear();
  internalPredicateSet.clear();

  // cleanning
  namedTypeHash = NULL;
  objectHashByType = NULL;
  objectHashByName = NULL;
  ramificationList = NULL;
  internalPredicateList = NULL;
  actionList = NULL;
  functionHash = NULL;
  predicateHash = NULL;
  arrayHash = NULL;
  problemClass *rv = declaredProblem;
  declaredProblem = NULL;
  return( rv );
}


static void
declareInit( const effectListClass *effect )
{
  assert( declaredProblem != NULL );
  declaredProblem->initEffect = effect;
}


static void
declareGoal( const formulaClass *formula )
{
  assert( declaredProblem != NULL );
  declaredProblem->goalFormula = formula;
}


static void
declareHook( const hookClass *hook )
{
  assert( declaredProblem != NULL );
  declaredProblem->hookType |= hook->hookType;
  switch( hook->hookType )
    {
    case problemClass::MODEL_HOOK:
      declaredProblem->modelHook = hook;
      break;
    case problemClass::BELIEF_HOOK:
      declaredProblem->beliefHook = hook;
      break;
    case problemClass::HEURISTIC_HOOK:
      declaredProblem->heuristicHook = hook;
      break;
    }
}


static void
cleanEnvironment( map<const char*,const typeClass*,ltcasestr> *env )
{
  map<const char*,const typeClass*,ltcasestr>::iterator it;

  for( it = env->begin(); it != env->end(); ++it )
    {
      //      fprintf( stderr, "free of [%p]%s\n", (*it).first, (*it).first );
      free( (void*)(*it).first );
      delete (*it).second;
    }
  delete env;
}


static void
printEnvironment( void )
{
  list<map<const char*,const typeClass*,ltcasestr>*>::const_iterator it;
  map<const char*,const typeClass*,ltcasestr>::const_iterator it2;
  for( it = variableEnvironment.begin(); it != variableEnvironment.end(); ++it )
    {
      for( it2 = (*it)->begin(); it2 != (*it)->end(); ++it2 )
	cout << "variable = " << (*it2).first << ", type = " << typeid( (*it2).second ).name() << endl;
    }
}


static list<map<const char*,const typeClass*,ltcasestr>*> *
environmentClone( void )
{
  list<map<const char*,const typeClass*,ltcasestr>*> *result;
  list<map<const char*,const typeClass*,ltcasestr>*>::const_iterator it;
  map<const char*,const typeClass*,ltcasestr> *m;
  map<const char*,const typeClass*,ltcasestr>::const_iterator it2;

  result = new list<map<const char*,const typeClass*,ltcasestr>*>;
  for( it = variableEnvironment.begin(); it != variableEnvironment.end(); ++it )
    if( (*it)->size() > 0 )
      {
	m = new map<const char*,const typeClass*,ltcasestr>;
	for( it2 = (*it)->begin(); it2 != (*it)->end(); ++it2 )
	  m->insert( make_pair( strdup( (*it2).first ), (*it2).second->clone() ) );
	result->push_back( m );
      }

  return( result );
}
#line 1867 "y.tab.c"
/* allocate initial stack or double stack size, up to YYMAXDEPTH */
static int yygrowstack()
{
    int newsize, i;
    short *newss;
    YYSTYPE *newvs;

    if ((newsize = yystacksize) == 0)
        newsize = YYINITSTACKSIZE;
    else if (newsize >= YYMAXDEPTH)
        return -1;
    else if ((newsize *= 2) > YYMAXDEPTH)
        newsize = YYMAXDEPTH;
    i = yyssp - yyss;
    newss = yyss ? (short *)realloc(yyss, newsize * sizeof *newss) :
      (short *)malloc(newsize * sizeof *newss);
    if (newss == NULL)
        return -1;
    yyss = newss;
    yyssp = newss + i;
    newvs = yyvs ? (YYSTYPE *)realloc(yyvs, newsize * sizeof *newvs) :
      (YYSTYPE *)malloc(newsize * sizeof *newvs);
    if (newvs == NULL)
        return -1;
    yyvs = newvs;
    yyvsp = newvs + i;
    yystacksize = newsize;
    yysslim = yyss + newsize - 1;
    return 0;
}

#define YYABORT goto yyabort
#define YYREJECT goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR goto yyerrlab

#ifndef YYPARSE_PARAM
#if defined(__cplusplus) || __STDC__
#define YYPARSE_PARAM_ARG void
#define YYPARSE_PARAM_DECL
#else	/* ! ANSI-C/C++ */
#define YYPARSE_PARAM_ARG
#define YYPARSE_PARAM_DECL
#endif	/* ANSI-C/C++ */
#else	/* YYPARSE_PARAM */
#ifndef YYPARSE_PARAM_TYPE
#define YYPARSE_PARAM_TYPE void *
#endif
#if defined(__cplusplus) || __STDC__
#define YYPARSE_PARAM_ARG YYPARSE_PARAM_TYPE YYPARSE_PARAM
#define YYPARSE_PARAM_DECL
#else	/* ! ANSI-C/C++ */
#define YYPARSE_PARAM_ARG YYPARSE_PARAM
#define YYPARSE_PARAM_DECL YYPARSE_PARAM_TYPE YYPARSE_PARAM;
#endif	/* ANSI-C/C++ */
#endif	/* ! YYPARSE_PARAM */

int
yyparse (YYPARSE_PARAM_ARG)
    YYPARSE_PARAM_DECL
{
    register int yym, yyn, yystate;
#if YYDEBUG
    register const char *yys;

    if ((yys = getenv("YYDEBUG")))
    {
        yyn = *yys;
        if (yyn >= '0' && yyn <= '9')
            yydebug = yyn - '0';
    }
#endif

    yynerrs = 0;
    yyerrflag = 0;
    yychar = (-1);

    if (yyss == NULL && yygrowstack()) goto yyoverflow;
    yyssp = yyss;
    yyvsp = yyvs;
    *yyssp = yystate = 0;

yyloop:
    if ((yyn = yydefred[yystate])) goto yyreduce;
    if (yychar < 0)
    {
        if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, reading %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
    }
    if ((yyn = yysindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: state %d, shifting to state %d\n",
                    YYPREFIX, yystate, yytable[yyn]);
#endif
        if (yyssp >= yysslim && yygrowstack())
        {
            goto yyoverflow;
        }
        *++yyssp = yystate = yytable[yyn];
        *++yyvsp = yylval;
        yychar = (-1);
        if (yyerrflag > 0)  --yyerrflag;
        goto yyloop;
    }
    if ((yyn = yyrindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
        yyn = yytable[yyn];
        goto yyreduce;
    }
    if (yyerrflag) goto yyinrecovery;
#if defined(lint) || defined(__GNUC__)
    goto yynewerror;
#endif
yynewerror:
    yyerror("syntax error");
#if defined(lint) || defined(__GNUC__)
    goto yyerrlab;
#endif
yyerrlab:
    ++yynerrs;
yyinrecovery:
    if (yyerrflag < 3)
    {
        yyerrflag = 3;
        for (;;)
        {
            if ((yyn = yysindex[*yyssp]) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == YYERRCODE)
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: state %d, error recovery shifting\
 to state %d\n", YYPREFIX, *yyssp, yytable[yyn]);
#endif
                if (yyssp >= yysslim && yygrowstack())
                {
                    goto yyoverflow;
                }
                *++yyssp = yystate = yytable[yyn];
                *++yyvsp = yylval;
                goto yyloop;
            }
            else
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: error recovery discarding state %d\n",
                            YYPREFIX, *yyssp);
#endif
                if (yyssp <= yyss) goto yyabort;
                --yyssp;
                --yyvsp;
            }
        }
    }
    else
    {
        if (yychar == 0) goto yyabort;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, error recovery discards token %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
        yychar = (-1);
        goto yyloop;
    }
yyreduce:
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: state %d, reducing by rule %d (%s)\n",
                YYPREFIX, yystate, yyn, yyrule[yyn]);
#endif
    yym = yylen[yyn];
    yyval = yyvsp[1-yym];
    switch (yyn)
    {
case 1:
#line 335 "daeParser.y"
{
			 globalChunk = yyvsp[0].codeChunkList;
		       }
break;
case 2:
#line 341 "daeParser.y"
{
			 yyval.codeChunkList = yyvsp[-1].codeChunkList;
			 yyval.codeChunkList->push_back( yyvsp[0].codeChunk );
		       }
break;
case 3:
#line 346 "daeParser.y"
{
			 yyval.codeChunkList = yyvsp[-1].codeChunkList;
			 yyval.codeChunkList->push_back( yyvsp[0].codeChunk );
		       }
break;
case 4:
#line 351 "daeParser.y"
{
			 yyval.codeChunkList = new list<const codeChunkClass*>;
		       }
break;
case 5:
#line 357 "daeParser.y"
{
			 startDomain( yyvsp[-1].ident );
		       }
break;
case 6:
#line 361 "daeParser.y"
{
			 yyval.codeChunk = finishDomain();
		       }
break;
case 9:
#line 371 "daeParser.y"
{
			 declareModel();
		       }
break;
case 14:
#line 379 "daeParser.y"
{
			 declareExternal( yyvsp[-1].identList );
		       }
break;
case 19:
#line 391 "daeParser.y"
{
			 declareDynamics( DETERMINISTIC );
		       }
break;
case 20:
#line 395 "daeParser.y"
{
			 declareDynamics( NON_DETERMINISTIC );
		       }
break;
case 21:
#line 399 "daeParser.y"
{
			 declareDynamics( PROBABILISTIC );
		       }
break;
case 22:
#line 405 "daeParser.y"
{
			 declareFeedback( COMPLETE );
		       }
break;
case 23:
#line 409 "daeParser.y"
{
			 declareFeedback( PARTIAL );
		       }
break;
case 24:
#line 413 "daeParser.y"
{
			 declareFeedback( NULLF );
		       }
break;
case 27:
#line 423 "daeParser.y"
{
			 startPackageParse( yyvsp[-1].ident );
		       }
break;
case 29:
#line 430 "daeParser.y"
{
			 finishPackageParse();
		       }
break;
case 32:
#line 440 "daeParser.y"
{
			 yyvsp[0].identList->push_front( yyvsp[-1].ident );
			 declareNamedType( strdup( yyvsp[-1].ident ), yyvsp[0].identList );
		       }
break;
case 33:
#line 445 "daeParser.y"
{
			 list<const char*> *aList = new list<const char*>;
			 aList->push_front( yyvsp[0].ident );
			 declareNamedType( strdup( yyvsp[0].ident ), aList );
		       }
break;
case 34:
#line 453 "daeParser.y"
{
			 yyval.identList = yyvsp[-1].identList;
			 yyval.identList->push_back( yyvsp[0].ident );
		       }
break;
case 35:
#line 458 "daeParser.y"
{
			 yyval.identList = new list<const char*>;
			 yyval.identList->push_back( yyvsp[0].ident );
		       }
break;
case 36:
#line 465 "daeParser.y"
{
			 yyval.ident = yyvsp[0].ident;
		       }
break;
case 39:
#line 475 "daeParser.y"
{
			 declareFunction( yyvsp[-3].ident, yyvsp[-2].type, yyvsp[-1].type );
		       }
break;
case 42:
#line 485 "daeParser.y"
{
			 domainType = predicateClass::LIST_DOMAIN;
			 declarePredicate( yyvsp[-2].ident, new typeListClass( yyvsp[-1].typeList ) );
		       }
break;
case 43:
#line 492 "daeParser.y"
{
			 domainType = functionClass::SIMPLE_DOMAIN;
			 yyval.type = yyvsp[0].type;
		       }
break;
case 44:
#line 497 "daeParser.y"
{
			 domainType = functionClass::LIST_DOMAIN;
			 yyval.type = new typeListClass( yyvsp[-1].typeList );
		       }
break;
case 45:
#line 502 "daeParser.y"
{
			 /* xxxxxx: array in function range */
			 /* xxxxxx: not supported*/
			 domainType = functionClass::ARRAY_DOMAIN;
			 assert( 0 );
		       }
break;
case 46:
#line 511 "daeParser.y"
{
			 yyval.type = new simpleTypeClass( yyvsp[0].ident );
		       }
break;
case 47:
#line 515 "daeParser.y"
{
			 yyval.type = new integerTypeClass( yyvsp[0].integer.min, yyvsp[0].integer.max );
		       }
break;
case 48:
#line 519 "daeParser.y"
{
			 yyval.type = new booleanTypeClass();
		       }
break;
case 49:
#line 525 "daeParser.y"
{
			 yyval.integer = yyvsp[0].integer;
		       }
break;
case 50:
#line 531 "daeParser.y"
{
			 yyval.integer.min = yyvsp[-3].ident;
			 yyval.integer.max = yyvsp[-1].ident;
		       }
break;
case 52:
#line 542 "daeParser.y"
{
			 yyval.typeList = yyvsp[-1].typeList;
			 yyval.typeList->push_back( yyvsp[0].type );
		       }
break;
case 53:
#line 547 "daeParser.y"
{
			 yyval.typeList = new list<const typeClass*>;
			 yyval.typeList->push_back( yyvsp[0].type );
		       }
break;
case 54:
#line 554 "daeParser.y"
{
			 yyval.identList = yyvsp[-1].identList;
			 yyval.identList->push_back( yyvsp[0].ident );
		       }
break;
case 55:
#line 559 "daeParser.y"
{ 
			 yyval.identList = new list<const char*>;
			 yyval.identList->push_back( yyvsp[0].ident );
                       }
break;
case 60:
#line 574 "daeParser.y"
{
			 declareSimpleObjects( yyvsp[0].ident, yyvsp[-2].identList );
		       }
break;
case 61:
#line 578 "daeParser.y"
{
			 declareIntegerObjects( yyvsp[0].integer.min, yyvsp[0].integer.max, yyvsp[-2].identList );
                       }
break;
case 62:
#line 582 "daeParser.y"
{
			 declareBooleanObjects( yyvsp[-2].identList );
                       }
break;
case 63:
#line 586 "daeParser.y"
{
			 declareArrayObjects( yyvsp[-2].ident, yyvsp[0].type, yyvsp[-6].identList );
                       }
break;
case 64:
#line 592 "daeParser.y"
{
			 yyval.identList = yyvsp[-1].identList;
			 yyval.identList->push_back( yyvsp[0].ident );
		       }
break;
case 65:
#line 597 "daeParser.y"
{
			 yyval.identList = new list<const char*>;
			 yyval.identList->push_back( yyvsp[0].ident );
		       }
break;
case 69:
#line 611 "daeParser.y"
{
			 variableEnvironment.push_front( new map<const char*,const typeClass*,ltcasestr> );
		       }
break;
case 70:
#line 615 "daeParser.y"
{
			 currInternalPredicate = yyvsp[-2].ident;
			 startLogicalDef( yyvsp[-2].ident );
		       }
break;
case 71:
#line 620 "daeParser.y"
{
			 currInternalPredicate = NULL;
			 declareLogicalDef( yyvsp[-6].ident, yyvsp[-1].formula );
		       }
break;
case 74:
#line 631 "daeParser.y"
{
			 startRamification( yyvsp[0].ident );
			 variableEnvironment.push_front( new map<const char*,const typeClass*,ltcasestr> );
		       }
break;
case 76:
#line 640 "daeParser.y"
{
			 declareRamification( yyvsp[-1].formula );
		       }
break;
case 77:
#line 644 "daeParser.y"
{
			 declareRamification( new effectListClass( yyvsp[-1].effectList ) );
		       }
break;
case 82:
#line 659 "daeParser.y"
{
			 list<const char*>::const_iterator it;
			 for( it = yyvsp[-2].identList->begin(); it != yyvsp[-2].identList->end(); ++it )
			   {
			     declareVariable( *it, yyvsp[0].type->clone() );
			     parameterList.push_back( make_pair( strdup(*it), yyvsp[0].type->clone() ) );
			   }
			 delete yyvsp[-2].identList;
			 delete yyvsp[0].type;
		       }
break;
case 83:
#line 672 "daeParser.y"
{
			 yyval.identList = yyvsp[-1].identList;
			 yyval.identList->push_back( yyvsp[0].ident );
		       }
break;
case 84:
#line 677 "daeParser.y"
{
			 yyval.identList = new list<const char*>;
			 yyval.identList->push_back( yyvsp[0].ident );
		       }
break;
case 87:
#line 688 "daeParser.y"
{
			 startAction( yyvsp[0].ident );
			 variableEnvironment.push_front( new map<const char*,const typeClass*,ltcasestr> );
		       }
break;
case 88:
#line 693 "daeParser.y"
{
			 finishAction();
		       }
break;
case 93:
#line 707 "daeParser.y"
{
			 declareActionPrecondition( yyvsp[0].formula );
		       }
break;
case 94:
#line 711 "daeParser.y"
{
			 insideEffect = true;
		       }
break;
case 95:
#line 715 "daeParser.y"
{
			 insideEffect = false;
			 declareActionEffect( yyvsp[0].effect );
		       }
break;
case 96:
#line 720 "daeParser.y"
{
			 declareActionCost( yyvsp[0].cost );
			 if( declaredDomain->model == PLANNING )
			   warning( featureInModelWarning, ":cost" );
		       }
break;
case 97:
#line 726 "daeParser.y"
{
			 declareActionObservation( yyvsp[0].observation );
			 if( (declaredDomain->model != ND_POMDP) && (declaredDomain->model != POMDP) )
			   error( featureInModel, declaredDomain->model );
		       }
break;
case 100:
#line 739 "daeParser.y"
{
			 yyval.formula = new notFormulaClass( yyvsp[-1].formula );
		       }
break;
case 101:
#line 743 "daeParser.y"
{
			 yyval.formula = new andFormulaClass( yyvsp[-1].formulaList );
		       }
break;
case 102:
#line 747 "daeParser.y"
{
			 yyval.formula = new orFormulaClass( yyvsp[-1].formulaList );
		       }
break;
case 103:
#line 751 "daeParser.y"
{
			 yyval.formula = new ifFormulaClass( yyvsp[-2].formula, yyvsp[-1].formula );
		       }
break;
case 104:
#line 755 "daeParser.y"
{
			 yyval.formula = new iffFormulaClass( yyvsp[-2].formula, yyvsp[-1].formula );
		       }
break;
case 105:
#line 759 "daeParser.y"
{
			 /* if variable already bounded, declare error*/
			 list<map<const char*,const typeClass*,ltcasestr>*>::const_iterator it;
			 for( it = variableEnvironment.begin(); it != variableEnvironment.end(); ++it )
			   if( (*it)->find( yyvsp[-3].ident ) != (*it)->end() )
			     error( nameClash, yyvsp[-3].ident );

			 /* extend current environment*/
			 variableEnvironment.push_front( new map<const char*,const typeClass*,ltcasestr> );
			 declareVariable( strdup(yyvsp[-3].ident), new simpleTypeClass( strdup(yyvsp[-1].ident) ) );
		       }
break;
case 106:
#line 771 "daeParser.y"
{
			 yyval.formula = new existsFormulaClass( yyvsp[-6].ident, new simpleTypeClass(yyvsp[-4].ident), yyvsp[-1].formula, environmentClone(), currInternalPredicate, insideEffect, yyvsp[-3].boolean );
			 cleanEnvironment( variableEnvironment.front() );
			 variableEnvironment.pop_front();
		       }
break;
case 107:
#line 777 "daeParser.y"
{
			 /* if variable already bounded, declare error*/
			 list<map<const char*,const typeClass*,ltcasestr>*>::const_iterator it;
			 for( it = variableEnvironment.begin(); it != variableEnvironment.end(); ++it )
			   if( (*it)->find( yyvsp[-3].ident ) != (*it)->end() )
			     error( nameClash, yyvsp[-3].ident );

			 /* extend current environment*/
			 variableEnvironment.push_front( new map<const char*,const typeClass*,ltcasestr> );
			 declareVariable( strdup(yyvsp[-3].ident), new simpleTypeClass( strdup(yyvsp[-1].ident) ) );
		       }
break;
case 108:
#line 789 "daeParser.y"
{
			 yyval.formula = new forallFormulaClass( yyvsp[-6].ident, new simpleTypeClass(yyvsp[-4].ident), yyvsp[-1].formula, environmentClone(), currInternalPredicate, insideEffect, yyvsp[-3].boolean );
			 cleanEnvironment( variableEnvironment.front() );
			 variableEnvironment.pop_front();
		       }
break;
case 109:
#line 797 "daeParser.y"
{
			 yyval.boolean = true;
		       }
break;
case 110:
#line 801 "daeParser.y"
{
			 yyval.boolean = false;
		       }
break;
case 111:
#line 807 "daeParser.y"
{
			 yyval.formulaList = yyvsp[-1].formulaList;
			 yyval.formulaList->push_back( yyvsp[0].formula );
		       }
break;
case 112:
#line 812 "daeParser.y"
{
			 yyval.formulaList = new list<const formulaClass*>;
			 yyval.formulaList->push_back( yyvsp[0].formula );
		       }
break;
case 113:
#line 828 "daeParser.y"
{
			 yyval.formula = new atomFormulaClass( yyvsp[-2].ident, new termListClass( yyvsp[-1].termList ) );
		       }
break;
case 114:
#line 832 "daeParser.y"
{
			 yyval.formula = new equalFormulaClass( yyvsp[-2].term, yyvsp[-1].term );
		       }
break;
case 115:
#line 836 "daeParser.y"
{
			 yyval.formula = new inclusionFormulaClass( yyvsp[-4].term, yyvsp[-2].termList );
		       }
break;
case 122:
#line 853 "daeParser.y"
{
			 yyval.termList = yyvsp[-1].termList;
			 yyval.termList->push_back( yyvsp[0].term );
		       }
break;
case 123:
#line 858 "daeParser.y"
{
			 yyval.termList = new list<const termClass*>;
		       }
break;
case 124:
#line 864 "daeParser.y"
{
			 yyval.term = fetchVariableTerm( yyvsp[0].ident, NULL );
		       }
break;
case 125:
#line 868 "daeParser.y"
{
			 yyval.term = fetchObjectTerm( yyvsp[-1].ident, yyvsp[0].term );
		       }
break;
case 127:
#line 873 "daeParser.y"
{
			 yyval.term = new functionTermClass( yyvsp[-2].ident, new termListClass( yyvsp[-1].termList ) );
		       }
break;
case 128:
#line 877 "daeParser.y"
{
			 yyval.term = new functionTermClass( yyvsp[-2].sfunction, new termListClass( yyvsp[-1].termList ) );
		       }
break;
case 129:
#line 881 "daeParser.y"
{
			 if( !insideEffect )
			   error( badStarTerm );
			 yyval.term = new starTermClass( yyvsp[0].term, insideEffect );
                       }
break;
case 130:
#line 887 "daeParser.y"
{
			 /* if variable already bounded, declare error*/
			 list<map<const char*,const typeClass*,ltcasestr>*>::const_iterator it;
			 for( it = variableEnvironment.begin(); it != variableEnvironment.end(); ++it )
			   if( (*it)->find( yyvsp[-3].ident ) != (*it)->end() )
			     error( nameClash, yyvsp[-3].ident );

			 /* extend current environment*/
			 variableEnvironment.push_front( new map<const char*,const typeClass*,ltcasestr> );
			 declareVariable( strdup(yyvsp[-3].ident), new simpleTypeClass( strdup(yyvsp[-1].ident) ) );
		       }
break;
case 131:
#line 899 "daeParser.y"
{
			 yyval.term = new sumTermClass(yyvsp[-6].ident,new simpleTypeClass(yyvsp[-4].ident),yyvsp[-1].term,environmentClone(),insideEffect,yyvsp[-3].boolean);
			 cleanEnvironment( variableEnvironment.front() );
			 variableEnvironment.pop_front();
		       }
break;
case 132:
#line 905 "daeParser.y"
{
			 yyval.term = new conditionalTermClass( yyvsp[-3].formula, yyvsp[-2].term, yyvsp[-1].term );
                       }
break;
case 133:
#line 909 "daeParser.y"
{
			 yyval.term = new formulaTermClass( yyvsp[-1].formula );
                       }
break;
case 134:
#line 916 "daeParser.y"
{
			 yyval.term = yyvsp[-1].term;
		       }
break;
case 135:
#line 920 "daeParser.y"
{
			 yyval.term = NULL;
		       }
break;
case 136:
#line 926 "daeParser.y"
{
			 yyval.term = new integerTermClass( yyvsp[0].ident );
		       }
break;
case 137:
#line 930 "daeParser.y"
{
			 yyval.term = new booleanTermClass( yyvsp[0].ident );
		       }
break;
case 138:
#line 937 "daeParser.y"
{
			 list<const termClass*> *extra = new list<const termClass*>;
			 extra->push_back( new integerTermClass( yyvsp[0].ident ) );
			 yyval.sfunction = new specialFunctionClass( strdup("_plus"), 
                                                        plusFunctionDomain->clone(), 
                                                        plusFunctionRange->clone(),
							new termListClass( extra ) );
		       }
break;
case 139:
#line 946 "daeParser.y"
{
			 list<const termClass*> *extra = new list<const termClass*>;
			 extra->push_back( new integerTermClass( yyvsp[0].ident ) );
			 yyval.sfunction = new specialFunctionClass( strdup("_minus"), 
                                                        minusFunctionDomain->clone(), 
                                                        minusFunctionRange->clone(),
							new termListClass( extra ) );
		       }
break;
case 140:
#line 957 "daeParser.y"
{
			 yyval.ident = yyvsp[-1].ident;
		       }
break;
case 141:
#line 961 "daeParser.y"
{
			 yyval.ident = strdup("-1");
		       }
break;
case 142:
#line 967 "daeParser.y"
{
			 yyval.effect = new effectListClass( yyvsp[0].effectList );
		       }
break;
case 143:
#line 971 "daeParser.y"
{
			 probabilisticEffectClass *result = new probabilisticEffectClass( yyvsp[-1].probEffectList );
			 yyval.effect = result;
			 declaredAction->numResultingStates = yyvsp[-1].probEffectList->size();
			 if( !checkProbabilities( result ) )
			   ++declaredAction->numResultingStates;
			 if( (declaredDomain->model != MDP) && (declaredDomain->model != POMDP) )
			   error( featureInModel, declaredDomain->model );
		       }
break;
case 144:
#line 981 "daeParser.y"
{
			 yyval.effect = new nonDeterministicEffectClass( yyvsp[-1].effectList );
			 declaredAction->numResultingStates = yyvsp[-1].effectList->size();
			 if( (declaredDomain->model != CONFORMANT) && (declaredDomain->model != ND_MDP) &&
			     (declaredDomain->model != ND_POMDP) )
			   error( featureInModel, declaredDomain->model );
		       }
break;
case 145:
#line 992 "daeParser.y"
{
			 yyval.effectList = yyvsp[-1].effectList;
			 yyval.effectList->push_back( yyvsp[0].effect );
		       }
break;
case 146:
#line 997 "daeParser.y"
{
			 yyval.effectList = new list<const effectClass*>;
			 yyval.effectList->push_back( yyvsp[0].effect );
		       }
break;
case 148:
#line 1005 "daeParser.y"
{
			 yyval.effect = new whenEffectClass( yyvsp[-2].formula, new effectListClass( yyvsp[-1].effectList ) );
		       }
break;
case 149:
#line 1009 "daeParser.y"
{
			 /* if variable already bounded, declare error*/
			 list<map<const char*,const typeClass*,ltcasestr>*>::const_iterator it;
			 for( it = variableEnvironment.begin(); it != variableEnvironment.end(); ++it )
			   if( (*it)->find( yyvsp[-2].ident ) != (*it)->end() )
			     error( nameClash, yyvsp[-2].ident );

			 /* extend current environment*/
			 variableEnvironment.push_front( new map<const char*,const typeClass*,ltcasestr> );
			 declareVariable( strdup(yyvsp[-2].ident), new simpleTypeClass( strdup(yyvsp[0].ident) ) );
		       }
break;
case 150:
#line 1021 "daeParser.y"
{
			 yyval.effect = new forallEffectClass( yyvsp[-5].ident, new simpleTypeClass(yyvsp[-3].ident), new effectListClass(yyvsp[-1].effectList) );
			 cleanEnvironment( variableEnvironment.front() );
			 variableEnvironment.pop_front();
		       }
break;
case 151:
#line 1029 "daeParser.y"
{
			 yyval.effectList = yyvsp[-1].effectList;
			 yyval.effectList->push_back( yyvsp[0].effect );
		       }
break;
case 152:
#line 1034 "daeParser.y"
{
			 yyval.effectList = new list<const effectClass*>;
			 yyval.effectList->push_back( yyvsp[0].effect );
		       }
break;
case 153:
#line 1041 "daeParser.y"
{
			 yyval.effect = new termSetEffectClass( yyvsp[-2].term, yyvsp[-1].term );

			 /*
			 ** to get information about dirty functions, we use RTTI.
			 ** I don't like that but now it seems a good reason to used it ..
			 */
			 const functionTermClass *f = dynamic_cast<const functionTermClass*>( yyvsp[-2].term );
			 if( f != NULL )
			   {
			     if( declaredAction ) declaredAction->dirtyFunctions.insert( strdup(f->function->ident) );
			     if( currentRamification ) ramDirtyFunctions.insert( strdup(f->function->ident) );
			   }
		       }
break;
case 154:
#line 1056 "daeParser.y"
{
			 if( declaredAction ) declaredAction->dirtyPredicates.insert( strdup(yyvsp[-4].ident) );
			 if( currentRamification ) ramDirtyPredicates.insert( strdup(yyvsp[-4].ident) );
			 yyval.effect = new predicateSetEffectClass( yyvsp[-4].ident, new termListClass( yyvsp[-3].termList ), yyvsp[-1].term );
		       }
break;
case 155:
#line 1062 "daeParser.y"
{
			 if( declaredAction ) declaredAction->dirtyPredicates.insert( strdup(yyvsp[-4].ident) );
			 if( currentRamification ) ramDirtyPredicates.insert( strdup(yyvsp[-4].ident) );
			 yyval.effect = new predicateSetEffectClass( yyvsp[-4].ident, new termListClass( yyvsp[-3].termList ), yyvsp[-1].formula );
		       }
break;
case 156:
#line 1068 "daeParser.y"
{
			 yyval.effect = new printEffectClass( yyvsp[-1].term );
		       }
break;
case 157:
#line 1072 "daeParser.y"
{
			 yyval.effect = new printEffectClass( yyvsp[-1].formula );
		       }
break;
case 158:
#line 1076 "daeParser.y"
{
			 yyval.effect = new printStateEffectClass();
		       }
break;
case 159:
#line 1082 "daeParser.y"
{
			 const functionTermClass *f = dynamic_cast<const functionTermClass*>( yyvsp[-6].term );
			 if( f != NULL ) declaredProblem->dirtyFunctions.insert( strdup(f->function->ident) );
			 yyval.effect = new initTermSetEffectClass( yyvsp[-6].term, yyvsp[-3].termList, yyvsp[-1].formula );
		       }
break;
case 160:
#line 1088 "daeParser.y"
{
			 const functionTermClass *f = dynamic_cast<const functionTermClass*>( yyvsp[-4].term );
			 if( f != NULL ) declaredProblem->dirtyFunctions.insert( strdup(f->function->ident) );
			 yyval.effect = new initTermSetEffectClass( yyvsp[-4].term, yyvsp[-2].integer.min, yyvsp[-2].integer.max, yyvsp[-1].formula );
		       }
break;
case 161:
#line 1095 "daeParser.y"
{
			 yyval.effect = new initPredicateSetEffectClass( yyvsp[-8].ident, new termListClass( yyvsp[-7].termList ), yyvsp[-3].termList, yyvsp[-1].formula );
			 declaredProblem->dirtyPredicates.insert( strdup(yyvsp[-8].ident) );
		       }
break;
case 162:
#line 1100 "daeParser.y"
{
			 yyval.effect = new oneOfEffectClass( yyvsp[-1].effectList );
		       }
break;
case 163:
#line 1104 "daeParser.y"
{
			 yyval.effect = new formulaInitEffectClass( yyvsp[0].formula );
		       }
break;
case 164:
#line 1110 "daeParser.y"
{
			 yyval.formula = yyvsp[0].formula;
		       }
break;
case 165:
#line 1114 "daeParser.y"
{
			 yyval.formula = NULL;
		       }
break;
case 166:
#line 1121 "daeParser.y"
{
			 yyval.probEffectList = yyvsp[-1].probEffectList;
			 yyval.probEffectList->push_back( yyvsp[0].probEffect );
		       }
break;
case 167:
#line 1126 "daeParser.y"
{
			 yyval.probEffectList = new list<pair<const char*,const effectClass*>*>;
			 yyval.probEffectList->push_back( yyvsp[0].probEffect );
		       }
break;
case 168:
#line 1134 "daeParser.y"
{
			 yyval.probEffect = new pair<const char*,const effectClass*>(yyvsp[-2].ident,new effectListClass( yyvsp[-1].effectList ));
		       }
break;
case 169:
#line 1141 "daeParser.y"
{
			 yyval.effectList = yyvsp[-1].effectList;
			 yyval.effectList->push_back( yyvsp[0].effect );
		       }
break;
case 170:
#line 1146 "daeParser.y"
{
			 yyval.effectList = new list<const effectClass*>;
			 yyval.effectList->push_back( yyvsp[0].effect );
		       }
break;
case 171:
#line 1154 "daeParser.y"
{
			 yyval.effect = new effectListClass( yyvsp[-1].effectList );
		       }
break;
case 172:
#line 1160 "daeParser.y"
{
			 yyval.cost = new termCostClass( yyvsp[0].term );
		       }
break;
case 173:
#line 1195 "daeParser.y"
{
			 yyval.observation = new observationListClass( yyvsp[0].obsList );
		       }
break;
case 174:
#line 1199 "daeParser.y"
{
			 checkProbabilities( yyvsp[-1].probObsList );
			 if( (declaredDomain->model != POMDP) )
			   error( featureInModel, declaredDomain->model );
			 yyval.observation = new probabilisticObservationClass( yyvsp[-1].probObsList );
		       }
break;
case 175:
#line 1208 "daeParser.y"
{
			 yyval.obsList = yyvsp[-1].obsList;
			 yyval.obsList->push_back( yyvsp[0].observation );
		       }
break;
case 176:
#line 1213 "daeParser.y"
{
			 yyval.obsList = new list<const observationClass*>;
			 yyval.obsList->push_back( yyvsp[0].observation );
		       }
break;
case 177:
#line 1221 "daeParser.y"
{
			 yyval.observation = new formulaObservationClass( yyvsp[0].formula );
		       }
break;
case 178:
#line 1225 "daeParser.y"
{
			 yyval.observation = new termObservationClass( yyvsp[0].term );
		       }
break;
case 179:
#line 1229 "daeParser.y"
{
			 yyval.observation = new conditionalFormulaObservationClass( yyvsp[-2].formula, yyvsp[-1].formula );
		       }
break;
case 180:
#line 1233 "daeParser.y"
{
			 yyval.observation = new conditionalTermObservationClass( yyvsp[-2].formula, yyvsp[-1].term );
		       }
break;
case 181:
#line 1237 "daeParser.y"
{
			 /* if variable already bounded, declare error*/
			 list<map<const char*,const typeClass*,ltcasestr>*>::const_iterator it;
			 for( it = variableEnvironment.begin(); it != variableEnvironment.end(); ++it )
			   if( (*it)->find( yyvsp[-2].ident ) != (*it)->end() )
			     error( nameClash, yyvsp[-2].ident );

			 /* extend current environment*/
			 variableEnvironment.push_front( new map<const char*,const typeClass*,ltcasestr> );
			 declareVariable( strdup(yyvsp[-2].ident), new simpleTypeClass( strdup(yyvsp[0].ident) ) );
		       }
break;
case 182:
#line 1249 "daeParser.y"
{
			 yyval.observation = new vectorObservationClass( yyvsp[-5].ident, new simpleTypeClass( yyvsp[-3].ident ), yyvsp[-1].term, environmentClone() );
			 cleanEnvironment( variableEnvironment.front() );
			 variableEnvironment.pop_front();
		       }
break;
case 183:
#line 1258 "daeParser.y"
{
			 yyval.probObsList = yyvsp[-1].probObsList;
			 yyval.probObsList->push_back( yyvsp[0].probObs );
		       }
break;
case 184:
#line 1263 "daeParser.y"
{
			 yyval.probObsList = new list<pair<const char*,const observationListClass*>*>;
			 yyval.probObsList->push_back( yyvsp[0].probObs );
		       }
break;
case 185:
#line 1271 "daeParser.y"
{
			 yyval.probObs = new pair<const char*,const observationListClass*>(yyvsp[-2].ident,new observationListClass(yyvsp[-1].obsList));
		       }
break;
case 186:
#line 1284 "daeParser.y"
{
			 startProblem( yyvsp[-5].ident, yyvsp[-1].ident );
		       }
break;
case 187:
#line 1288 "daeParser.y"
{
			 yyval.codeChunk = finishProblem();
			 
			 /* can't wait to the end for code generation */
			 generateCode( (problemClass*)yyval.codeChunk );
		       }
break;
case 194:
#line 1307 "daeParser.y"
{
			 declareInit( new effectListClass( yyvsp[0].effectList ) );
		       }
break;
case 195:
#line 1313 "daeParser.y"
{
			 declareGoal( yyvsp[0].formula );
		       }
break;
case 196:
#line 1317 "daeParser.y"
{
			 declareGoal( new certaintyGoalClass );
		       }
break;
case 197:
#line 1324 "daeParser.y"
{
			 yyval.effectList = yyvsp[-1].effectList;
			 yyval.effectList->push_back( yyvsp[0].effect );
		       }
break;
case 198:
#line 1329 "daeParser.y"
{
			 yyval.effectList = new list<const effectClass*>;
			 yyval.effectList->push_back( yyvsp[0].effect );
		       }
break;
case 201:
#line 1346 "daeParser.y"
{
			 yyval.effect = new whenEffectClass( yyvsp[-2].formula, new effectListClass( yyvsp[-1].effectList ) );
		       }
break;
case 202:
#line 1350 "daeParser.y"
{
			 /* if variable already bounded, declare error*/
			 list<map<const char*,const typeClass*,ltcasestr>*>::const_iterator it;
			 for( it = variableEnvironment.begin(); it != variableEnvironment.end(); ++it )
			   if( (*it)->find( yyvsp[-2].ident ) != (*it)->end() )
			     error( nameClash, yyvsp[-2].ident );

			 /* extend current environment*/
			 variableEnvironment.push_front( new map<const char*,const typeClass*,ltcasestr> );
			 declareVariable( strdup(yyvsp[-2].ident), new simpleTypeClass( strdup(yyvsp[0].ident) ) );
		       }
break;
case 203:
#line 1362 "daeParser.y"
{
			 yyval.effect = new forallEffectClass( yyvsp[-5].ident, new simpleTypeClass(yyvsp[-3].ident), new effectListClass(yyvsp[-1].effectList) );
			 cleanEnvironment( variableEnvironment.front() );
			 variableEnvironment.pop_front();
		       }
break;
case 204:
#line 1370 "daeParser.y"
{
			 declareHook( yyvsp[0].hook );
		       }
break;
case 205:
#line 1374 "daeParser.y"
{
			 declareHook( yyvsp[0].hook );
		       }
break;
case 206:
#line 1380 "daeParser.y"
{
			 yyval.hook = new hookClass( problemClass::MODEL_HOOK, yyvsp[-1].ident );
		       }
break;
case 207:
#line 1384 "daeParser.y"
{
			 yyval.hook = new hookClass( problemClass::BELIEF_HOOK, yyvsp[-1].ident );
		       }
break;
case 208:
#line 1388 "daeParser.y"
{
			 yyval.hook = new hookClass( problemClass::HEURISTIC_HOOK, yyvsp[-1].ident );
		       }
break;
case 209:
#line 1400 "daeParser.y"
{
			 startPackage( yyvsp[0].ident );
		       }
break;
case 210:
#line 1404 "daeParser.y"
{
			 finishPackage();
		       }
break;
#line 3137 "y.tab.c"
    }
    yyssp -= yym;
    yystate = *yyssp;
    yyvsp -= yym;
    yym = yylhs[yyn];
    if (yystate == 0 && yym == 0)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: after reduction, shifting from state 0 to\
 state %d\n", YYPREFIX, YYFINAL);
#endif
        yystate = YYFINAL;
        *++yyssp = YYFINAL;
        *++yyvsp = yyval;
        if (yychar < 0)
        {
            if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
            if (yydebug)
            {
                yys = 0;
                if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
                if (!yys) yys = "illegal-symbol";
                printf("%sdebug: state %d, reading %d (%s)\n",
                        YYPREFIX, YYFINAL, yychar, yys);
            }
#endif
        }
        if (yychar == 0) goto yyaccept;
        goto yyloop;
    }
    if ((yyn = yygindex[yym]) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yystate)
        yystate = yytable[yyn];
    else
        yystate = yydgoto[yym];
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: after reduction, shifting from state %d \
to state %d\n", YYPREFIX, *yyssp, yystate);
#endif
    if (yyssp >= yysslim && yygrowstack())
    {
        goto yyoverflow;
    }
    *++yyssp = yystate;
    *++yyvsp = yyval;
    goto yyloop;
yyoverflow:
    yyerror("yacc stack overflow");
yyabort:
    return (1);
yyaccept:
    return (0);
}
