//  Theseus
//  theMemoryMgmt.cc -- Memory Management Implemetantion
//
//  Blai Bonet, Hector Geffner
//  Universidad Simon Bolivar, 1998, 1999, 2000, 2001, 2002

#if 0 // Using Purify
#ifndef NDEBUG
#include <stdio.h>
#include <stl.h>


static map<void*,size_t> memoryAllocations;


void *
operator new( size_t size )
{
  void *ptr;
  ptr = malloc(size);
  memoryAllocations.insert( make_pair( ptr, size ) );
#if 0
  if( size == 16 )
    fprintf( stderr, "operator new: %p(%d)\n", ptr, size );
#endif
  return( ptr );
}


void
operator delete( void *ptr )
{
  if( ptr )
    {
      map<void*,size_t>::iterator it;
      if( (it = memoryAllocations.find( ptr )) == memoryAllocations.end() )
	fprintf( stderr, "ERROR: bad argument to delete: %p\n", ptr );
      else
	{
#if 0
	  if( (*it).second == 16 )
	    fprintf( stderr, "operator delete: %p(%d)\n", ptr, (*it).second );
#endif
	  memoryAllocations.erase( it );
	  free( ptr );
	}
    }
}


void *
operator new[]( size_t size )
{
  void *ptr;
  ptr = malloc(size);
  memoryAllocations.insert( make_pair( ptr, size ) );
#if 0
  if( size == 24 )
    fprintf( stderr, "operator new[]: %p(%d)\n", ptr, size );
#endif
  return( ptr );
}


void
operator delete[]( void *ptr )
{
  if( ptr )
    {
      map<void*,size_t>::iterator it;
      if( (it = memoryAllocations.find( ptr )) == memoryAllocations.end() )
	fprintf( stderr, "ERROR: bad argument to delete[]: %p\n", ptr );
      else
	{
#if 0
	  if( (*it).second == 16 )
	    fprintf( stderr, "operator delete[]: %p(%d)\n", ptr, (*it).second );
#endif
	  memoryAllocations.erase( it );
	  free( ptr );
	}
    }
}


void
memoryReport( bool flag )
{
  map<void*,size_t>::const_iterator it;
  size_t sum;

  sum = 0;
  for( it = memoryAllocations.begin(); it != memoryAllocations.end(); ++it )
    {
      sum += (*it).second;
      if( flag )
	cerr << "leak at " << (*it).first << " of size " << (*it).second << endl;
    }
  cerr << "total leak: " << sum << " bytes" << endl;
}
#endif







#if 0
#include <theseus/theGlobal.h>
#include <theseus/theMemoryMgmt.h>


///////////////////////////////////////////////////////////////////////////////
//
// Memory Management Class
//

memoryMgmtClass::memoryMgmtClass( const memoryMgmtClass& memory )
{
  assert( 0 );
}


memoryMgmtClass::memoryMgmtClass()
{
  memset( freeList, 0, sizeInitialTable * sizeof( memory_t * ) );
  memoryAreaTable = new memoryAreaTable_t[sizeInitialTable];
  sizeMemoryAreaTable = sizeInitialTable;
  numberMemoryAreas = 0;
  claimSize = initialClaimSize;
  mode = standardMode;
}


memoryMgmtClass::~memoryMgmtClass( void )
{
  register int i;

  for( i = 0; i < numberMemoryAreas; ++i )
    delete[] memoryAreaTable[i].base;
  delete[] memoryAreaTable;
}


char *
memoryMgmtClass::allocateFromFreeList( size_t size )
{
  unsigned index;
  memory_t *memory;

  index = getIndex( size );
  if( freeList[index] != NULL )
    {
      // update free list
      memory = freeList[index];
      freeList[index] = memory->next;
      memory->next = NULL;
      return( memory->chunk );
    }
  return( NULL );
}


char *
memoryMgmtClass::allocateFromPool( size_t size )
{
  register int  i;
  memory_t      *memory;

  // look for a memory area
  for( i = 0; i < numberMemoryAreas; ++i )
    if( (memoryAreaTable[i].index + size + sizeof( memory_t * ) + sizeof( int )) <= memoryAreaTable[i].boundary )
      break;

  // check if we need another memory area
  if( i == numberMemoryAreas )
    getNewMemoryArea( size + sizeof( memory_t * ) + sizeof( int ), i );

  // setup result
  memory = (memory_t*)memoryAreaTable[i].index;
  memory->size = size;
  memory->next = NULL;
  memory->chunk = (char*)&memory->chunk;
  memoryAreaTable[i].index += size + sizeof( memory_t * ) + sizeof( int );
  return( memory->chunk );
}


void
memoryMgmtClass::getNewMemoryArea( size_t size, int &i )
{
  allocationMode    oldMode;
  memoryAreaTable_t *newMemoryAreaTable;

  // change allocation mode to standardMode
  oldMode = mode;
  mode = standardMode;

  // check if we need to increase space for memory areas
  if( numberMemoryAreas == sizeMemoryAreaTable )
    {
      newMemoryAreaTable = new memoryAreaTable_t[2 * sizeMemoryAreaTable];
      memcpy( newMemoryAreaTable, memoryAreaTable, 2 * sizeMemoryAreaTable * sizeof( memoryAreaTable_t ) );
      delete[] memoryAreaTable;
      memoryAreaTable = newMemoryAreaTable;
      sizeMemoryAreaTable *= 2;
    }

  // compute new claimSize
  do {
    claimSize = claimSize * 2;
  } while( size > claimSize );

  // get the memory
  memoryAreaTable[numberMemoryAreas].base = new char[claimSize];
  memoryAreaTable[numberMemoryAreas].index = memoryAreaTable[numberMemoryAreas].base;
  memoryAreaTable[numberMemoryAreas].boundary = memoryAreaTable[numberMemoryAreas].base + claimSize;
  i = numberMemoryAreas++;

  // restore allocation mode
  mode = oldMode;
}


int
memoryMgmtClass::getIndex( size_t size )
{
  register unsigned i, bit;
  
  if( size == 0 )
    {
      return( -1 );
    }
  else
    {
      bit = 1;
      for( i = 0; i < (8 * sizeof( size_t )); ++i, bit = bit << 1 )
	if( (size & ~bit) == 0 )
	  break;
      return( i < (8 * sizeof( size_t )) ? (int)i : -1 );
    }
}


void
memoryMgmtClass::setAllocationMode( allocationMode newMode )
{
  mode = newMode;
}


char *
memoryMgmtClass::allocate( size_t size )
{
  char *result;

  assert( goodSize( size ) );

  // check if there is some chunk in the free list, else
  // get a new chunk from the memory pool
  if( (result = allocateFromFreeList( size )) == NULL )
    result = allocateFromPool( size );
  return( result );
}


void
memoryMgmtClass::deallocate( char *ptr )
{
  memory_t  *memory;
  unsigned  index;

  // restore size
  memory = (memory_t*)(ptr - sizeof( memory_t * ) - sizeof( int ));
  memory->chunk = ptr;
  assert( memory->chunk == (char*)&(memory->chunk) );
  assert( goodSize( memory->size ) );
  assert( memory->next == NULL );

  // deallocate chunk
  index = getIndex( memory->size );
  memory->next = freeList[index];
  freeList[index] = memory;
}


bool
memoryMgmtClass::goodSize( size_t size )
{
  register unsigned i, bit;
  
  if( size == 0 )
    {
      return( false );
    }
  else
    {
      bit = 1;
      for( i = 0; i < (8 * sizeof( size_t )); ++i, bit = bit << 1 )
	if( (size & ~bit) == 0 )
	  break;
      return( i < (8 * sizeof( size_t )) );
    }
}


void
memoryMgmtClass::clean( void )
{
  register int i;

  for( i = 0; i < numberMemoryAreas; ++i )
    delete[] memoryAreaTable[i].base;
  delete[] memoryAreaTable;

  memset( freeList, 0, sizeInitialTable * sizeof( memory_t * ) );
  memoryAreaTable = new memoryAreaTable_t[sizeInitialTable];
  sizeMemoryAreaTable = sizeInitialTable;
  numberMemoryAreas = 0;
  claimSize = initialClaimSize;
}
#endif
#endif
