//  Theseus
//  theQuantization.cc -- Belief Quantization
//
//  Blai Bonet, Hector Geffner
//  Universidad Simon Bolivar, 1998, 1999, 2000, 2001, 2002

#include <iostream>
#include <map>
#include <math.h>

using namespace std;
#include <theseus/theTopLevel.h>
#include <theseus/theQuantization.h>
#include <theseus/theStandardBelief.h>
#include <theseus/theBeliefCache.h>


///////////////////////////////////////////////////////////////////////////////
//
// Quatization Cached Data Class
//

class quantizationCachedDataClass
{
public:
  // members
  hashEntryClass* quantization;

  // constructors
  quantizationCachedDataClass();

  // destructors
  ~quantizationCachedDataClass();

  // static methods
  static quantizationCachedDataClass* constructor( void );
  static void                         destructor( void *cachedData );
};

quantizationCachedDataClass::quantizationCachedDataClass()
{
  quantization = NULL;
}

quantizationCachedDataClass::~quantizationCachedDataClass()
{
}

quantizationCachedDataClass*
quantizationCachedDataClass::constructor( void )
{
  return( new quantizationCachedDataClass );
}

void
quantizationCachedDataClass::destructor( void *cachedData )
{
  delete (quantizationCachedDataClass*)cachedData;
}


///////////////////////////////////////////////////////////////////////////////
//
// Quantization Class
//

// structure
struct pair_s
{
  int   idx;
  float val;
};

// static data
static float*                x = NULL;
static standardBeliefClass*  result = NULL;


// static members
int                          quantizationClass::dimensions = 0;
float                        quantizationClass::levels = 0.0;
beliefCacheClass*            quantizationClass::cache = NULL;


quantizationClass::quantizationClass()
{
}

quantizationClass::~quantizationClass()
{
}

void
quantizationClass::initialize( int pLevels, int pDimensions )
{
  // set parameters
  levels = (float)pLevels;
  dimensions = pDimensions;

  // allocate working space
  x = new float[dimensions];
  result = new standardBeliefClass;
}

void
quantizationClass::setCache( int cacheSize, int clusterSize,
			     beliefClass* (*beliefConstructor)( void ) )
{
  delete cache;
  cache = new beliefCacheClass( "quantizationCache", 
				cacheSize,
				clusterSize, 
				beliefConstructor,
				(void *(*)()) &quantizationCachedDataClass::constructor,
				&quantizationCachedDataClass::destructor );
}

void
quantizationClass::finalize( void )
{
  delete[] x;
  delete result;
  delete cache;
  x = NULL;
  result = NULL;
  cache = NULL;
}

void
quantizationClass::clean( void )
{
  if( cache )
    cache->clean();
}

void
quantizationClass::quantize( beliefHashClass* beliefHash,
			     bool useCache,
			     bool saveInHash,
			     const standardBeliefClass* belief, 
			     hashEntryClass* &quantization )
{
  register float val, sum;
  register hashEntryClass* entry;
  register quantizationCachedDataClass* cachedBel;
  const map<int,float>* m;
  map<int,float>::const_iterator it;
  static hashEntryClass localQuantization;

  // cache lookup
  if( useCache && ((cachedBel = (quantizationCachedDataClass*)cache->lookup( belief, 0 )) != NULL) )
    {
      // return data. O(1).
      quantization = cachedBel->quantization;
      return;
    }
  else if( saveInHash )
    {
      // evict someone from cache
      cachedBel = (quantizationCachedDataClass*)cache->freeSlot( belief, 0 );
    }

  if( (PD.version == problemClass::VERSION_GRID) || (PD.version == problemClass::VERSION_ORIGINAL) )
    {
#if 0
      // compute un-normalized belief
      sum = 0.0;
      m = belief->cast();
      memset( x, 0, dimensions * sizeof( float ) );
      for( it = m->begin(); it != m->end(); ++it )
	{
	  val = (*it).second;
	  if( val == 0.0 )
	    {
	      x[(*it).first] = 0.0;
	    }
	  else
	    {
	      val = (float)floor( 1.5 + (double)(val * levels) );
	      x[(*it).first] = (val == 0.0 ? (10000.0 * PD.epsilon) : val);
	    }
	  sum += x[(*it).first];
	}

      // normalize
      for( it = m->begin(); it != m->end(); ++it )
	x[(*it).first] /= sum;

      // fill
      result->fill( x, dimensions );
      assert( result->check() );
    }
  else if( RTDPversion == ORIGINAL )
    {
#else
      // compute un-normalized belief
      sum = 0.0;
      m = belief->cast();
      for( it = m->begin(); it != m->end(); ++it )
	sum += (float)floor(1.5+(double)(levels*(*it).second));

      // normalize 
      result->clean();
      for( it = m->begin(); it != m->end(); ++it )
	result->insert( (*it).first, (float)(floor(1.5+(double)(levels*(*it).second))/sum) );
      assert( result->check() );
    }
#endif

  //xxxx cout << *belief << " ==> " << *result << endl;

  // populate belief hash
  if( (entry = beliefHash->lookup( (void*)result )) == NULL )
    {
      if( saveInHash )
	{
	  entry = beliefHash->insert( (void*)result, result->heuristicValue(), (void*)false );
	  cachedBel->quantization = entry;
	  quantization = entry;
	}
      else
	{
	  localQuantization.setValue( result->heuristicValue() );
	  quantization = &localQuantization;
	}
    }
  else
    {
      if( saveInHash )
	cachedBel->quantization = entry;
      quantization = entry;
    }
}
