//  Theseus
//  theResult.cc -- Result Implementation
//
//  Blai Bonet, Hector Geffner
//  Universidad Simon Bolivar, 1998, 1999, 2000, 2001, 2002

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include <iostream>

using namespace std;
#include <theseus/theGlobal.h>
#include <theseus/theResult.h>
#include <theseus/theUtils.h>


///////////////////////////////////////////////////////////////////////////////
//
// Result Class
//

resultClass::resultClass( const resultClass& result )
{
}

resultClass::resultClass( void )
{
  numMoves = -1;
  secs = (unsigned long)-1;
  usecs = (unsigned long)-1;
}

resultClass::~resultClass( void )
{
}

void
resultClass::startTimer( void )
{
  gettimeofday( &startTime, NULL );
}

void
resultClass::stopTimer( void )
{
  gettimeofday( &stopTime, NULL );
}

unsigned long
resultClass::getElapsedSeconds( void )
{
  if( secs == (unsigned long)-1 )
    diffTime( secs, usecs, startTime, stopTime );
  return( secs );
}

unsigned long
resultClass::getElapsedUSeconds( void )
{
  if( usecs == (unsigned long)-1 )
    diffTime( secs, usecs, startTime, stopTime );
  return( usecs );
}


///////////////////////////////////////////////////////////////////////////////
//
// Standard Result Class
//

standardResultClass::standardResultClass( const standardResultClass& result )
{
}

standardResultClass::standardResultClass()
{
}

standardResultClass::~standardResultClass( void )
{
  clean();
}

void
standardResultClass::clean( void )
{
  secs = (unsigned long)-1;
  usecs = (unsigned long)-1;
  stepList.clear();
}

void
standardResultClass::print( ostream& os, int outputLevel, problemHandleClass *pHandle )
{
  stepClass firstStep = stepList.front();
  stepClass lastStep = stepList.back();
  list<stepClass>::const_iterator it;

  os << "<e=" << (float)getElapsedSeconds() + ((float)getElapsedUSeconds()) / 1000000.0 << ">:";
  os << "<r=" << run << ">:";
  os << "<t=" << (runType == 1 ? 'l' : 'c') << ">:";
  os << "<v=" << initialValue << ">:";
  os << "<l=" << (solved ? 's' : 'u') << ">:";
  os << "<i=" << firstStep.state << ">:";
  os << "<f=" << lastStep.state << ">:";

  switch( outputLevel )
    {
    case 0:
      os << "<s=" << numMoves << ">:<c=" << costToGoal << ">:<d=" << discountedCostToGoal << ">";
      break;
    case 1:
      os << "<s=" << numMoves << ">:<c=" << costToGoal << ">:<d=" << discountedCostToGoal << ">:";
      if( numMoves != -1 )
	{
	  os << "<p=";
	  for( it = stepList.begin(); it != stepList.end(); )
	    {
	      if( pHandle != NULL )
		{
		  os << '<';
		  (*pHandle->printAction)( os, (*it).action );
		  os << ",obs" << (*it).observation;
		}
	      else
		{
		  os << "<act" << (*it).action << ",obs" << (*it).action;
		}
	      ++it;
	      os << (it != stepList.end() ? ">," : ">");
	    }
	  os << ">";
	}
      else
	os << "<p=null>";
      break;
    }
  os << endl;
}

void
standardResultClass::push_back( int state, int action, int observation )
{
  stepClass step( state, action, observation );
  stepList.push_back( step );
  ++numMoves;
}

void
standardResultClass::push_front( int state, int action, int observation )
{
  stepClass step( state, action, observation );
  stepList.push_front( step );
  ++numMoves;
}

void
standardResultClass::pop_back( void )
{
  stepList.pop_back();
  --numMoves;
}

void
standardResultClass::pop_front( void )
{
  stepList.pop_front();
  --numMoves;
}


///////////////////////////////////////////////////////////////////////////////
//
// Search Result Class
//

searchResultClass::searchResultClass()
  : standardResultClass()
{
}


searchResultClass::~searchResultClass()
{
  clean();
}

void
searchResultClass::print( ostream& os, int outputLevel, problemHandleClass *pHandle )
{
  list<stepClass>::const_iterator it;

  os << "<e=" << (float)getElapsedSeconds() + ((float)getElapsedUSeconds()) / 1000000.0 << ">:";
  os << "<r=" << run << ">:";
  os << "<t=" << (runType == 1 ? 'l' : 'c') << ">:";
  os << "<v=" << initialValue << ">:";

  switch( outputLevel )
    {
    case 0:
      os << "<s=" << numMoves << ">:<c=" << costToGoal << ">:<d=" << discountedCostToGoal << ">";
      break;
    case 1:
      os << "<s=" << numMoves << ">:<c=" << costToGoal << ">:<d=" << discountedCostToGoal << ">:";
      if( numMoves != -1 )
	{
	  os << "<p=<";
	  assert( pHandle != NULL );
	  for( it = stepList.begin(); it != stepList.end(); )
	    {
	      (*pHandle->printAction)( os, (*it).action );
	      ++it;
	      os << (it != stepList.end() ? "," : "");
	    }
	  os << ">>";
	}
      else
	os << "<p=null>";
      break;
    }
  os << endl;

  // print sequence of actions plan-like
  os << "%plan-begin" << endl;
  for( it = stepList.begin(); it != stepList.end(); ++it )
    {
      os << "%step ";
      (*pHandle->printAction)( os, (*it).action );
      os << endl;
    }
  os << "%plan-end" << endl;
}
