//  Theseus
//  theRtHash.cc -- RunTime State and Observation Hash
//
//  Blai Bonet, Hector Geffner
//  Universidad Simon Bolivar, 1998, 1999, 2000, 2001, 2002

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include <iostream>

using namespace std;
#include <theseus/theRtHash.h>
#include <theseus/theRtStandard.h>
#include <theseus/md4.h>


/////////////////////////////////////////////////////////////////////////////////////////
//
// stateHashClass
//

stateHashClass::stateHashClass( void )
  : hashClass( 1024 )
{
  size = 0;
  buff = NULL;
}


stateHashClass::stateHashClass( const stateHashClass& hash )
  : hashClass( 1024 )
{
  *this = hash;
}


stateHashClass::~stateHashClass( void )
{
  clean();
  delete[] buff;
}


void
stateHashClass::setSize( int pSize )
{
  size = pSize;
  buff = new char[size];
}


unsigned
stateHashClass::dataFunction( const void *data )
{
  register unsigned *ptr, result;
  unsigned char digest[16];
  MD4_CTX context;

  // copy belief into buffer
  memcpy( buff, data, size * sizeof( char ) );

  // compute MD4 digests
  MD4Init( &context );
  MD4Update( &context, (unsigned char*)buff, size );
  MD4Final( digest, &context );

  // compact digest into an unsigned and return it (this assumes that sizeof( unsigned ) = 4)
  ptr = (unsigned*)digest;
  result = (ptr[0] ^ ptr[1] ^ ptr[2] ^ ptr[3]);
  return( result );
}


void *
stateHashClass::dataCopy( const void *data )
{
  return( ((stateClass *)data)->clone() );
}


bool
stateHashClass::dataCompare( const void *data1, const void *data2 )
{
  return( !(((stateClass *)data1)->operator==( *((stateClass *)data2) )) );
}


void
stateHashClass::dataDelete( void *data )
{
  delete( (stateClass *)data );
}


void
stateHashClass::dataPrint( ostream& os, const void *data )
{
  ((stateClass *)data)->print( os, 0 );
}


hashClass*
stateHashClass::constructor( void )
{
  return( new stateHashClass );
}


void*
stateHashClass::dataRead( istream& is )
{
  // xxxxxxx
  return( NULL );
}


void
stateHashClass::dataWrite( ostream& os, const void *data ) const
{
}


/////////////////////////////////////////////////////////////////////////////////////////
//
// invStateHashClass
//

invStateHashClass::invStateHashClass( void )
  : hashClass( 1024 )
{
}


invStateHashClass::invStateHashClass( const invStateHashClass& hash )
  : hashClass( 1024 )
{
  *this = hash;
}


invStateHashClass::~invStateHashClass( void )
{
  clean();
}


unsigned
invStateHashClass::dataFunction( const void *data )
{
  return( (unsigned)data );
}


void*
invStateHashClass::dataCopy( const void *data )
{
  return( (void*)data );
}


bool
invStateHashClass::dataCompare( const void *data1, const void *data2 )
{
  return( !((unsigned)data1 == (unsigned)data2) );
}


void
invStateHashClass::dataDelete( void *data )
{
}


void
invStateHashClass::dataPrint( ostream& os, const void *data )
{
  os << (unsigned)data;
}


hashClass*
invStateHashClass::constructor( void )
{
  return( new invStateHashClass );
}


void*
invStateHashClass::dataRead( istream& is )
{
  // xxxxxxx
  return( NULL );
}


void
invStateHashClass::dataWrite( ostream& os, const void *data ) const
{
}


/////////////////////////////////////////////////////////////////////////////////////////
//
// observationHashClass
//


observationHashClass::observationHashClass( void )
  : hashClass( 1024 )
{
  size = 0;
  buff = NULL;
}


observationHashClass::observationHashClass( const observationHashClass& hash )
  : hashClass( 1024 )
{
  *this = hash;
}


observationHashClass::~observationHashClass( void )
{
  clean();
  delete[] buff;
}


void
observationHashClass::setSize( int pSize )
{
  size = pSize;
  buff = new char[size];
}


unsigned
observationHashClass::dataFunction( const void *data )
{
  register unsigned *ptr, result;
  unsigned char digest[16];
  MD4_CTX context;

  // copy belief into buffer
  memcpy( buff, data, size * sizeof( char ) );

  // compute MD4 digests
  MD4Init( &context );
  MD4Update( &context, (unsigned char*)buff, size );
  MD4Final( digest, &context );

  // compact digest into an unsigned and return it (this assumes that sizeof( unsigned ) = 4)
  ptr = (unsigned*)digest;
  result = (ptr[0] ^ ptr[1] ^ ptr[2] ^ ptr[3]);
  return( result );
}


void *
observationHashClass::dataCopy( const void *data )
{
  return( ((observationClass *)data)->clone() );
}


bool
observationHashClass::dataCompare( const void *data1, const void *data2 )
{
  return( !(((observationClass *)data1)->operator==( *((observationClass *)data2) )) );
}


void
observationHashClass::dataDelete( void *data )
{
  delete( (observationClass *)data );
}


void
observationHashClass::dataPrint( ostream& os, const void *data )
{
  ((observationClass *)data)->print( os, 0 );
}


hashClass*
observationHashClass::constructor( void )
{
  return( new observationHashClass );
}


void*
observationHashClass::dataRead( istream& is )
{
  // xxxxxxx
  return( NULL );
}


void
observationHashClass:: dataWrite( ostream& os, const void *data ) const
{
}



/////////////////////////////////////////////////////////////////////////////////////////
//
// invObservationHashClass
//

invObservationHashClass::invObservationHashClass( void )
  : hashClass( 1024 )
{
}


invObservationHashClass::invObservationHashClass( const invObservationHashClass& hash )
  : hashClass( 1024 )
{
  *this = hash;
}


invObservationHashClass::~invObservationHashClass( void )
{
  clean();
}


unsigned
invObservationHashClass::dataFunction( const void *data )
{
  return( (unsigned)data );
}


void*
invObservationHashClass::dataCopy( const void *data )
{
  return( (void*)data );
}


bool
invObservationHashClass::dataCompare( const void *data1, const void *data2 )
{
  return( !((unsigned)data1 == (unsigned)data2) );
}


void
invObservationHashClass::dataDelete( void *data )
{
}


void
invObservationHashClass::dataPrint( ostream& os, const void *data )
{
  os << (unsigned)data;
}


hashClass*
invObservationHashClass::constructor( void )
{
  return( new invObservationHashClass );
}


void*
invObservationHashClass::dataRead( istream& is )
{
  // xxxxxxx
  return( NULL );
}


void
invObservationHashClass::dataWrite( ostream& os, const void *data ) const
{
}
