//  Theseus
//  theSearchBelief.cc -- Search Belief Implementation
//
//  Blai Bonet, Hector Geffner
//  Universidad Simon Bolivar, 1998, 1999, 2000, 2001, 2002

#include <iostream>
#include <typeinfo>

using namespace std;
//#include <theseus/theGlobal.h>
#include <theseus/theSearchBelief.h>
#include <theseus/theBeliefCache.h>
#include <theseus/theSearchModel.h>
#include <theseus/theHeuristic.h>
#include <theseus/theUtils.h>
#include <theseus/md4.h>


///////////////////////////////////////////////////////////////////////////////
//
// Search Belief Class
//

// static members
searchModelClass* searchBeliefClass::model = NULL;


searchBeliefClass::searchBeliefClass()
{
}

searchBeliefClass::searchBeliefClass( const searchBeliefClass& belief )
{
  *this = belief;
}

searchBeliefClass::~searchBeliefClass( void )
{
}

void
searchBeliefClass::setModel( modelClass* m )
{
  model = dynamic_cast<searchModelClass*>( m );
}

const searchModelClass*
searchBeliefClass::getModel( void ) const
{
  return( model );
}

beliefClass::constructor_t
searchBeliefClass::getConstructor( void ) const
{
  return( (beliefClass *(*)(void)) &searchBeliefClass::constructor );
}

void
searchBeliefClass::print( ostream& os ) const
{
  set<int>::const_iterator it;

  os << "[ ";
  for( it = bel.begin(); it != bel.end(); ++it )
    os << *it << " ";
  os << "]";
}

bool
searchBeliefClass::check( void ) const
{
  return( bel.size() > 0 );
}

bool
searchBeliefClass::check( int state ) const
{
  return( bel.find( state ) != bel.end() );
}

const map<int,float>*
searchBeliefClass::cast( void ) const
{
  set<int>::const_iterator it;
  static map<int,float> result;

  result.erase( result.begin(), result.end() );
  for( it = bel.begin(); it != bel.end(); ++it )
    result.insert( make_pair( *it, (float)(1.0/bel.size()) ) );
  return( &result );
}

void
searchBeliefClass::insert( int state, float probability )
{
  bel.insert( state );
}

unsigned
searchBeliefClass::randomSampling( void ) const
{
  return( ::randomSampling( bel ) );
}

void
searchBeliefClass::checkModelAvailability( modelClass* m ) const
{
  static set<int>::const_iterator it;
  
  for( it = bel.begin(); it != bel.end(); ++it )
    m->checkModelFor( *it );
}

void
searchBeliefClass::nextPossibleObservations( int action, map<int,float> &result ) const
{
  cerr << "error: nextPossibleObservations called" << endl;
}

int
searchBeliefClass::plausibleState( int action, int observation ) const
{
  cerr << "error: plausibleState called" << endl;
  return( -1 );
}

beliefClass*
searchBeliefClass::updateByA( beliefCacheClass *cache, bool useCache, int action ) const
{
  unsigned              tag;
  searchBeliefClass     *bel_a; 
  beliefCachedDataClass *cachedBel;
  set<int>              workingBel;
  set<int>::const_iterator it1;
  map<int,float>::const_iterator it2;
  map<int,map<int,float> >::const_iterator handle;

  // cache lookup
  tag = action;
  if( useCache && ((cachedBel = (beliefCachedDataClass*)cache->lookup( this, tag )) != NULL ) )
    {
      return( cachedBel->belief );
    }
  else
    {
      // we need to evict someone from the cache belief
      cachedBel = (beliefCachedDataClass*)cache->freeSlot( this, tag );
      bel_a = (searchBeliefClass*)cachedBel->belief;
    }
  assert( bel_a != this );

  // calculation only for those bel_a(s) different from zero (see docs)
  for( it1 = bel.begin(); it1 != bel.end(); ++it1 )
    {
      assert( (model->transitionModel[action].find( *it1 ) != model->transitionModel[action].end()) && 
      	      !model->transitionModel[action][*it1].empty() );
      handle = model->transitionModel[action].find( *it1 );
      for( it2 = (*handle).second.begin(); it2 != (*handle).second.end(); ++it2 )
	workingBel.insert( (*it2).first );
    }

  // return
  bel_a->bel = workingBel;
  return( bel_a );
}

beliefClass*
searchBeliefClass::updateByAO( beliefCacheClass *cache, bool useCache, int action, int observation ) const
{
  cerr << "error searchBeliefClass::updateByAO() called" << endl;
  return( NULL );
}

beliefClass*  
searchBeliefClass::clone( void ) const
{
  return( new searchBeliefClass( *this ) );
}

float
searchBeliefClass::heuristicValue( void ) const
{
  if( heuristic != NULL )
    return( heuristic->value( this ) );
  else
    return( 0.0 );
}

void
searchBeliefClass::support( set<int> &result ) const
{
  set<int>::const_iterator it;
  result.clear();
  for( it = bel.begin(); it != bel.end(); ++it )
    result.insert( *it );
}

int
searchBeliefClass::supportSize( void ) const
{
  return( bel.size() );
}

void
searchBeliefClass::clean( void )
{
  bel.erase( bel.begin(), bel.end() );
}

beliefClass&
searchBeliefClass::operator=( const beliefClass& pBel )
{
  *this = (searchBeliefClass&)pBel;
  return( *this );
}

beliefClass& 
searchBeliefClass::operator=( const searchBeliefClass& pBel )
{
  bel = pBel.bel;
  return( *this );
}

bool
searchBeliefClass::operator==( const beliefClass& pBel ) const
{
  return( *this == (searchBeliefClass&) pBel );
}

bool
searchBeliefClass::operator==( const searchBeliefClass& pBel ) const
{
  return( (bel == pBel.bel) );
}

unsigned
searchBeliefClass::hashDataFunction( void ) const
{
  register unsigned *ptr, result;
  unsigned char digest[16];

  static MD4_CTX context;
  static unsigned buffSize = 0;
  static unsigned *buff = NULL;
  static set<int>::const_iterator it;

  // space allocation (if needed)
  if( bel.size() > buffSize )
    {
      for( ; bel.size() > buffSize; buffSize = (buffSize == 0 ? 64 : 2 * buffSize) );
      if( buff != NULL )
	delete[] buff;
      buff = new unsigned[buffSize];
    }

  // copy belief into buffer
  ptr = buff;
  for( it = bel.begin(); it != bel.end(); ++it )
    {
      // this assumes that sizeof( float ) = sizeof( int )
      *ptr++ = (unsigned)*it;
    }

  // compute MD4 digests
  MD4Init( &context );
  MD4Update( &context, (unsigned char*)buff, 4*(ptr - buff) );
  MD4Final( digest, &context );

  // compact digest into an unsigned and return it (this assumes that sizeof( unsigned ) = 4)
  ptr = (unsigned*)digest;
  result = (ptr[0] ^ ptr[1] ^ ptr[2] ^ ptr[3]);
  return( result );
}

void *
searchBeliefClass::hashDataCopy( void ) const
{
  return( (void*)clone() );
}

bool
searchBeliefClass::hashDataCompare( const void *data ) const
{
  return( !(bel == ((searchBeliefClass*)data)->bel) );
}

void
searchBeliefClass::hashDataWrite( ostream& os ) const
{
  write( os );
}

void *
searchBeliefClass::hashDataRead( istream& is ) const
{
  return( beliefClass::read( is ) );
}


///////////////////////
//
// serialization

void
searchBeliefClass::checkIn( void )
{
  signRegistration( typeid( searchBeliefClass ).name(),
		    (void (*)( istream&, beliefClass * )) &searchBeliefClass::fill,
		    (beliefClass *(*)( void )) &searchBeliefClass::constructor );
}

void
searchBeliefClass::write( ostream& os ) const
{
  const char *id;
  unsigned len;

  // write registrationId
  id = typeid( *this ).name();
  len = strlen( id ) + 1;
  safeWrite( &len, sizeof( unsigned ), 1, os );
  safeWrite( (void*)id, sizeof( char ), strlen( id ) + 1, os );

  // serialize base class
  beliefClass::write( os );

  // do it!
  os << bel;
}

searchBeliefClass *
searchBeliefClass::read( istream& is )
{
  searchBeliefClass *bel;

  bel = new searchBeliefClass;
  fill( is, bel );

  return( bel );
}

searchBeliefClass *
searchBeliefClass::constructor( void )
{
  return( new searchBeliefClass() );
}

void
searchBeliefClass::fill( istream& is, beliefClass *bel )
{
  fill( is, (searchBeliefClass*)bel );
}

void
searchBeliefClass::fill( istream& is, searchBeliefClass *bel )
{
  beliefClass::fill( is, bel );
  is >> bel->bel;
}


///////////////////////////////////////////////////////////////////////////////
//
// Search Belief Hash Class
//

searchBeliefHashClass::searchBeliefHashClass( void )
{
}

searchBeliefHashClass::searchBeliefHashClass( const searchBeliefHashClass& hash )
{
  *this = hash;
}

searchBeliefHashClass::~searchBeliefHashClass( void )
{
  clean();
}

void
searchBeliefHashClass::quantize( bool useCache, bool saveInHash, const beliefClass* belief,
				 hashEntryClass*& quantization )
{
  cerr << "warning: searchBeliefHashClass::quantize() called" << endl;
}

unsigned 
searchBeliefHashClass::dataFunction( const void *data )
{
  return( ((searchBeliefClass*)data)->hashDataFunction() );
}

void *
searchBeliefHashClass::dataCopy( const void *data )
{
  return( ((searchBeliefClass*)data)->hashDataCopy() );
}

bool
searchBeliefHashClass::dataCompare( const void *newData, const void *dataInHash )
{
  return( !(*((searchBeliefClass*)newData) == *((searchBeliefClass*)dataInHash)) );
}

void
searchBeliefHashClass::dataDelete( void *data )
{
  delete (searchBeliefClass*)data;
}

void
searchBeliefHashClass::dataPrint( ostream& os, const void *data )
{
  os << "[ " << *((searchBeliefClass*)data) << "] ";
} 

void
searchBeliefHashClass::dataWrite( ostream& os, const void *data ) const
{
  ((searchBeliefClass*)data)->write( os );
}

void *
searchBeliefHashClass::dataRead( istream& is )
{
  return( beliefClass::read( is ) );
}

void
searchBeliefHashClass::statistics( ostream& os )
{
  hashClass::statistics( os );
}


///////////////////////
//
// serialization

void
searchBeliefHashClass::checkIn( void )
{
  signRegistration( typeid( searchBeliefHashClass ).name(),
		    (void (*)( istream&, hashClass * )) &searchBeliefHashClass::fill,
		    (hashClass *(*)( void )) &searchBeliefHashClass::constructor );
}

void
searchBeliefHashClass::write( ostream& os ) const
{
  const char *id;
  unsigned len;

  // write registrationId
  id = typeid( *this ).name();
  len = strlen( id ) + 1;
  safeWrite( &len, sizeof( unsigned ), 1, os );
  safeWrite( (void*)id, sizeof( char ), strlen( id ) + 1, os );

  // serialize base class
  beliefHashClass::write( os );
}

searchBeliefHashClass *
searchBeliefHashClass::read( istream& is )
{
  searchBeliefHashClass *result;

  result = new searchBeliefHashClass;
  fill( is, result );
  return( result );
}

searchBeliefHashClass *
searchBeliefHashClass::constructor( void )
{
  return( new searchBeliefHashClass );
}

void
searchBeliefHashClass::fill( istream& is, searchBeliefHashClass *hash )
{
  beliefHashClass::fill( is, hash );
}
