//  Theseus
//  theSearchModel.cc -- Search Model Implementation
//
//  Blai Bonet, Hector Geffner
//  Universidad Simon Bolivar, 1998, 1999, 2000, 2001, 2002

#include <iostream>
#include <typeinfo>

using namespace std;
#include <theseus/theTopLevel.h>
#include <theseus/theSearchModel.h>
#include <theseus/theSearchBelief.h>
#include <theseus/theRtStandard.h>


///////////////////////////////////////////////////////////////////////////////
//
// Search Model Class
//

searchModelClass::searchModelClass()
{
}

searchModelClass::searchModelClass( const searchModelClass& model )
{
}

searchModelClass::~searchModelClass( void )
{
}

float
searchModelClass::cost( const beliefClass *belief, int action ) const
{
  register float sum;
  const set<int>* s;
  set<int>::const_iterator it;

  assert( belief != NULL );

  s = &((searchBeliefClass*)belief)->bel;
  sum = 0.0;
  for( it = s->begin(); it != s->end(); ++it )
    sum += standardModelClass::cost( *it, action );
  sum /= (float)s->size();
  return( sum );
}

bool
searchModelClass::applicable( const beliefClass* belief, int action ) const
{
  const set<int>* s;
  set<int>::const_iterator it;

  assert( belief != NULL );

  s = &((searchBeliefClass*)belief)->bel;
  for( it = s->begin(); it != s->end(); ++it )
    if( !standardModelClass::applicable( *it, action ) )
      return( false );
  return( true );
}

bool
searchModelClass::inGoal( const beliefClass* belief ) const
{
  assert( belief != NULL );
  if( PD.pddlProblemType && (PD.handle->otherInfo & (1<<problemHandleClass::CERTAINTY_GOAL)) )
    {
      return( belief->supportSize() == 1 );
    }
  else
    {
      const set<int>* s;
      set<int>::const_iterator it;

      s = &((searchBeliefClass*)belief)->bel;
      for( it = s->begin(); it != s->end(); ++it )
	if( !standardModelClass::inGoal( *it ) )
	  return( false );
      return( true );
    }
}


///////////////////////
//
// serialization

void
searchModelClass::checkIn( void )
{
  signRegistration( typeid( searchModelClass ).name(),
		    (void (*)( istream&, modelClass * )) &searchModelClass::fill,
		    (modelClass* (*)( void )) &searchModelClass::constructor );
}

searchModelClass*
searchModelClass::read( istream& is )
{
  searchModelClass *model;

  model = new searchModelClass;
  fill( is, model );
  return( model );
}

searchModelClass*
searchModelClass::constructor( void )
{
  return( new searchModelClass );
}

void
searchModelClass::fill( istream& is, searchModelClass *model )
{
  standardModelClass::fill( is, model );
}
