//  Theseus
//  theSearchPOMDP.h -- Search POMDP Implementation
//
//  Blai Bonet, Hector Geffner
//  Universidad Simon Bolivar, 1998, 1999, 2000, 2001, 2002

#ifndef _theSearchPOMDP_INCLUDE_
#define _theSearchPOMDP_INCLUDE_

#include <iostream>

#include <theseus/thePOMDP.h>


///////////////////////////////////////////////////////////////////////////////
//
// Search POMDP Class
//

class searchPOMDPClass : public POMDPClass
{
private:
  // private methods
  searchPOMDPClass( const searchPOMDPClass& pomdp );

  // constants
  enum { SEARCH_IDA, SEARCH_BFS };

  // private virtual methods
  virtual void              algorithm( bool learning, resultClass& result, int extraParameter );

public:
  // static members
  static int                numberStates;

  // constructors/destructors
  searchPOMDPClass();
  virtual ~searchPOMDPClass( void );

  // static methods
  static void               setup( int numberStates );

  // virtual methods
  virtual void              setTheInitialBelief( const beliefClass *belief );
  virtual float             cost( int state, int action ) const;
  virtual float             cost( const beliefClass *belief, int action ) const;
  virtual bool              inGoal( int state ) const;
  virtual bool              inGoal( const beliefClass* belief ) const;
  virtual bool              applicable( int state, int action ) const;
  virtual bool              applicable( const beliefClass* belief, int action ) const;
  virtual int               getInitialState( const beliefClass* belief ) const;
  virtual int               nextState( int state, int action ) const;
  virtual int               nextObservation( int state, int action ) const;
  virtual hashValueClass*   hashValue( const beliefClass* belief );

  // serialization
  static  void              checkIn( void );
  virtual void              write( ostream& os ) const;
  static searchPOMDPClass*  read( istream& is );
  static searchPOMDPClass*  constructor( void );
  static void               fill( istream& is, searchPOMDPClass *pomdp );
};


#endif // _theSearchPOMDP_INCLUDE
