//  Theseus
//  theSimpleBelief.h -- Simple Belief Implementation
//
//  Blai Bonet, Hector Geffner
//  Universidad Simon Bolivar, 1998, 1999, 2000, 2001, 2002

#ifndef _theSimpleBelief_INCLUDE_
#define _theSimpleBelief_INCLUDE_

#include <iostream>
#include <map>

#include <theseus/theBelief.h>


class standardModelClass;


///////////////////////////////////////////////////////////////////////////////
//
// Simple Belief Class
//

class simpleBeliefClass : public beliefClass
{
private:
  // members
  int state;

  // static members
  static standardModelClass *model;

  // private methods
  simpleBeliefClass( const simpleBeliefClass& belief );

  // private virtual methods
  virtual void print( ostream& os ) const;

public:
  // constructors/destructors
  simpleBeliefClass();
  simpleBeliefClass( int s );
  virtual ~simpleBeliefClass( void );

  // methods
  void                          setState( int s ) { state = s; }
  int                           getState( void )  { return( state ); } 

  // virtual methods
  virtual void                  setModel( modelClass* model );
  virtual beliefClass::constructor_t getConstructor( void ) const;
  virtual bool                  check( void ) const;
  virtual bool                  check( int state ) const;
  virtual const map<int,float>* cast( void ) const;
  virtual void                  insert( int state, float probability );
  virtual void                  checkModelAvailability( modelClass* model ) const;
  virtual unsigned              randomSampling( void ) const;
  virtual void                  nextPossibleObservations( int action, map<int,float> &result ) const;
  virtual int                   plausibleState( int action, int observation ) const;
  virtual beliefClass*          updateByA( beliefCacheClass* cache, bool useCache, int action ) const;
  virtual beliefClass*          updateByAO( beliefCacheClass* cache, bool useCache, int action, int observation ) const;
  virtual beliefClass*          clone( void ) const;
  virtual float                 heuristicValue( void ) const;
  virtual void                  support( set<int> &result ) const;
  virtual int                   supportSize( void ) const;

  // operator overload
  virtual beliefClass&          operator=( const beliefClass& bel );
  virtual beliefClass&          operator=( const simpleBeliefClass& bel );
  virtual bool                  operator==( const beliefClass& bel ) const;
  virtual bool                  operator==( const simpleBeliefClass& bel ) const;

  // hash virtual methods
  virtual unsigned              hashDataFunction( void ) const;
  virtual void*                 hashDataCopy( void ) const;
  virtual bool                  hashDataCompare( const void* data ) const;
  virtual void                  hashDataWrite( ostream& os ) const;
  virtual void*                 hashDataRead( istream& is ) const;

  // serialization
  static  void                  checkIn( void );
  virtual void                  write( ostream& os ) const;
  static  simpleBeliefClass*    read( istream& is );
  static  simpleBeliefClass*    constructor( void );
  static  void                  fill( istream& is, beliefClass* bel );
  static  void                  fill( istream& is, simpleBeliefClass* bel );

  // friends
  friend class simpleBeliefHashClass;
};


///////////////////////////////////////////////////////////////////////////////
//
// Simple Belief Hash Class
//

class simpleBeliefHashClass : public beliefHashClass
{
  // private methods
  simpleBeliefHashClass( const simpleBeliefHashClass& hash );

public:
  // constructors/destructors
  simpleBeliefHashClass();
  virtual ~simpleBeliefHashClass( void );

  // virtual hash functions (since this is a hashClass derived class)
  virtual void                     clean( void );
  virtual unsigned                 dataFunction( const void* data );
  virtual void*                    dataCopy( const void* data );
  virtual bool                     dataCompare( const void* newData, const void* dataInHash );
  virtual void                     dataDelete( void* data );
  virtual void                     dataPrint( ostream& os, const void* data );
  virtual void                     dataWrite( ostream& os, const void* data ) const;
  virtual void*                    dataRead( istream& is );
  virtual void                     statistics( ostream& os );

  // virtual quantization functions
  virtual void                     quantize( bool useCache, bool saveInHash, const beliefClass* belief, 
					     hashEntryClass*& quantization );

  // serialization
  static  void                     checkIn( void );
  virtual void                     write( ostream& os ) const;
  static  simpleBeliefHashClass*   read( istream& is );
  static  simpleBeliefHashClass*   constructor( void );
  static  void                     fill( istream& is, simpleBeliefHashClass* hash );
};


#endif // _theSimpleBelief_INCLUDE
