//  Theseus
//  theTopLevel.h -- Top Level Routines
//
//  Blai Bonet, Hector Geffner
//  Universidad Simon Bolivar, 1998, 1999, 2000, 2001, 2002

#ifndef _theTopLevel_INCLUDE_
#define _theTopLevel_INCLUDE_

#if 0
#include <theseus/theGlobal.h>
#include <theseus/theException.h>
#include <theseus/theMemoryMgmt.h>
#include <theseus/theUtils.h>

#include <theseus/theHeuristic.h>
#include <theseus/theQmdpHeuristic.h>

#include <theseus/theBelief.h>
#include <theseus/theSimpleBelief.h>
#include <theseus/theStandardBelief.h>
#include <theseus/thePlanningBelief.h>
#include <theseus/theSearchBelief.h>

#include <theseus/theQuantization.h>

#include <theseus/theModel.h>
#include <theseus/theStandardModel.h>
#include <theseus/thePlanningModel.h>
#include <theseus/theSearchModel.h>

#include <theseus/thePOMDP.h>
#include <theseus/theStandardPOMDP.h>
#include <theseus/thePlanningPOMDP.h>
#include <theseus/theSearchPOMDP.h>
#include <theseus/theMDPPOMDP.h>

#include <theseus/theRtStandard.h>
#include <theseus/theRtRegister.h>
#endif

#include <set>


class POMDPClass;
class modelClass;
class beliefClass;
class heuristicClass;
class lookAheadHeuristicClass;
class problemHandleClass;


#define  MAX_PIMS          128


class problemClass 
{
public:
  // members
  int                      signal;

  char*                    programName;
  char*                    problemFile;
  char*                    outputPrefix;
  char*                    outputFilename;
  char*                    coreFilename;
  char*                    linkmap;
  ostream*                 outputFile;

  const char*              softwareRevision;
  int                      version;
  int                      cutoff;
  int                      runs;
  int                      pimSteps[MAX_PIMS];
  int                      learningTrials[MAX_PIMS];
  int                      controlTrials[MAX_PIMS];
  int                      discretizationLevels;
  int                      extraParameter;
  float                    discountFactor;
  float                    MDPdiscountFactor;
  int                      lookahead;
  int                      incrementalModel;
  bool                     zeroHeuristic;
  int                      randomSeed;
  bool                     randomTies;
  bool                     pddlProblemType;
  int                      outputLevel;
  int                      precision;
  int                      coreLevel;
  int                      loadType;         // not yet implemented
  int                      cacheSize[3];
  int                      clusterSize[3];

  bool                     useStopRule;
  double                   SREpsilon;
  double                   epsilon;

  POMDPClass*              POMDP;
  modelClass*              model;
  beliefClass*             belief;
  lookAheadHeuristicClass* heuristic;
  heuristicClass*          baseHeuristic;
  problemHandleClass*      handle;

  // methods
  void clean( int level );

  // constants
  enum { CLEAN_OBJECT, CLEAN_CORE, CLEAN_HASH };
  enum { VERSION_GRID, VERSION_ORIGINAL };
};


// global data
extern problemClass          PD;


// global functions
void parseArguments( int argc, char **argv, void (*helpFunction)() );
void internalInitialization( void );
void internalFinalization( void );
void setOutputFormat( ostream& os );
void printParameters( ostream& os, const char *prefix );
int  bootstrapProblem( const char *workingDir, const char *entryPoint );
void writeCore( ostream& os );
int  readCore( const char *coreFilename );
void solveProblem( void );



///////////////////////////////////////////////////////////////////////////////
//
// Policy Generation
//


typedef pair<int,const beliefClass*> policyGraphNodeClass;
typedef pair<int,pair<policyGraphNodeClass,policyGraphNodeClass> > policyGraphEdgeClass;

class policyGraphClass
{
private:
  void generatePolicyGraphAux( policyGraphNodeClass& node );

public:
  set<policyGraphNodeClass> nodes;
  set<policyGraphEdgeClass> edges;
  set<policyGraphNodeClass> goals;
  set<policyGraphNodeClass> roots;

  ~policyGraphClass();
  const policyGraphNodeClass& retrieveNode( const policyGraphNodeClass& node );
  void                        generatePolicyGraph( void );
  void                        outputPolicyGraph( ostream& out );
};

typedef pair<int,const beliefClass*> policyTableNodeClass;

class policyTableClass
{
private:
  void generatePolicyTableAux( policyTableNodeClass& node );

public:
  set<policyTableNodeClass> nodes;
  set<policyTableNodeClass> goals;
  set<int>                  states;

  ~policyTableClass();
  bool inserted( const policyTableNodeClass& node );
  void generatePolicyTable( void );
  void outputPolicyTable( ostream& out );
};


#endif // _theTopLevel_INCLUDE
